@@ -167,15 +167,15 @@ int main(int argc, char *argv[])
         options.RowPerm = LargeDiag_MC64;
         options.ReplaceTinyPivot = NO;
         options.Trans = NOTRANS;
-        options.IterRefine = DOUBLE;
+        options.IterRefine = SLU_DOUBLE;
         options.SolveInitialized = NO;
         options.RefineInitialized = NO;
         options.PrintStat = YES;
      */
     set_default_options_dist(&options);
+    options.IterRefine = SLU_SINGLE;
 
     if (!iam) {
-	print_sp_ienv_dist(&options);
 	print_options_dist(&options);
 	fflush(stdout);
     }
