@@ -18,33 +18,30 @@
 #include "superlu_ddefs.h"
 
 #ifdef GPU_ACC // enable GPU
-
+#include "gpu_api_utils.h"
 // #include "mkl.h"
-
-#include <cublas_v2.h>
-#include <cuda_runtime.h>
 // #include "sec_structs.h"
 // #include "supernodal_etree.h"
 
 /* Constants */
 //#define SLU_TARGET_GPU 0
 //#define MAX_BLOCK_SIZE 10000
-#define MAX_NCUDA_STREAMS 32
+#define MAX_NGPU_STREAMS 32
 
 static
-void check(cudaError_t result, char const *const func, const char *const file, int const line)
+void check(gpuError_t result, char const *const func, const char *const file, int const line)
 {
     if (result)
     {
-        fprintf(stderr, "CUDA error at file %s: line %d code=(%s) \"%s\" \n",
-                file, line, cudaGetErrorString(result), func);
+        fprintf(stderr, "GPU error at file %s: line %d code=(%s) \"%s\" \n",
+                file, line, gpuGetErrorString(result), func);
 
-        // Make sure we call CUDA Device Reset before exiting
+        // Make sure we call GPU Device Reset before exiting
         exit(EXIT_FAILURE);
     }
 }
 
-#define checkCudaErrors(val)  check ( (val), #val, __FILE__, __LINE__ )
+#define checkGPUErrors(val)  check ( (val), #val, __FILE__, __LINE__ )
 
 typedef struct //SCUbuf_gpu_
 {
@@ -103,7 +100,7 @@ typedef struct //LUstruct_gpu_
     int_t *ijb_lookupPtr;
 
     // GPU buffers for performing Schur Complement Update on GPU
-    dSCUbuf_gpu_t scubufs[MAX_NCUDA_STREAMS];
+    dSCUbuf_gpu_t scubufs[MAX_NGPU_STREAMS];
     double *acc_L_buff, *acc_U_buff;
 
     /*Informations for various buffers*/
@@ -112,6 +109,7 @@ typedef struct //LUstruct_gpu_
     int_t *xsup;
     gridinfo_t *grid;
 
+#if 0 // Sherry: moved to 'SuperLUStat_t'
     double ScatterMOPCounter;
     double ScatterMOPTimer;
     double GemmFLOPCounter;
@@ -122,12 +120,13 @@ typedef struct //LUstruct_gpu_
     double tHost_PCIeH2D;
     double tHost_PCIeD2H;
 
-    /*cuda events to measure DGEMM and SCATTER timing */
+    /*GPU events to measure DGEMM and SCATTER timing */
     int *isOffloaded;  /*stores if any iteration is offloaded or not*/
-    cudaEvent_t *GemmStart, *GemmEnd, *ScatterEnd;  /*cuda events to store gemm and scatter's begin and end*/
-    cudaEvent_t *ePCIeH2D;
-    cudaEvent_t *ePCIeD2H_Start;
-    cudaEvent_t *ePCIeD2H_End;
+    gpuEvent_t *GemmStart, *GemmEnd, *ScatterEnd;  /*GPU events to store gemm and scatter's begin and end*/
+    gpuEvent_t *ePCIeH2D;
+    gpuEvent_t *ePCIeD2H_Start;
+    gpuEvent_t *ePCIeD2H_End;
+#endif
 
     int_t *xsup_host;
     int_t* perm_c_supno;
@@ -136,12 +135,12 @@ typedef struct //LUstruct_gpu_
 
 typedef struct //sluGPU_t_
 {
-    int_t gpuId;        // if there are multiple GPUs
+    //int gpuId;      // if there are multiple GPUs ( NOT USED )
     dLUstruct_gpu_t *A_gpu, *dA_gpu; // holds the LU structure on GPU
-    cudaStream_t funCallStreams[MAX_NCUDA_STREAMS], CopyStream;
-    cublasHandle_t cublasHandles[MAX_NCUDA_STREAMS];
-    int_t lastOffloadStream[MAX_NCUDA_STREAMS];
-    int_t nCudaStreams;
+    gpuStream_t funCallStreams[MAX_NGPU_STREAMS], CopyStream;
+    gpublasHandle_t gpublasHandles[MAX_NGPU_STREAMS];
+    int lastOffloadStream[MAX_NGPU_STREAMS];
+    int nGPUStreams;
     int* isNodeInMyGrid;
     double acc_async_cost;
 } dsluGPU_t;
@@ -190,7 +189,8 @@ extern int dreduceGPUlu(int last_flag, d2Hreduce_t* d2Hred,
 	 dLUstruct_t *LUstruct);
 
 extern int dwaitGPUscu(int streamId, dsluGPU_t *sluGPU, SCT_t *SCT);
-extern int dsendLUpanelGPU2HOST( int_t k0, d2Hreduce_t* d2Hred, dsluGPU_t *sluGPU);
+extern int dsendLUpanelGPU2HOST( int_t k0, d2Hreduce_t* d2Hred,
+       	   dsluGPU_t *sluGPU, SuperLUStat_t *);
 extern int dsendSCUdataHost2GPU(
     int_t streamId, int_t* lsub, int_t* usub, double* bigU, int_t bigu_send_size,
     int_t Remain_lbuf_send_size,  dsluGPU_t *sluGPU, HyP_t* HyP
@@ -200,7 +200,8 @@ extern int dinitSluGPU3D_t(
     dsluGPU_t *sluGPU,
     dLUstruct_t *LUstruct,
     gridinfo3d_t * grid3d,
-    int_t* perm_c_supno, int_t n, int_t buffer_size, int_t bigu_size, int_t ldt
+    int_t* perm_c_supno, int_t n, int_t buffer_size, int_t bigu_size, int_t ldt,
+    SuperLUStat_t *
 );
 int dSchurCompUpdate_GPU(
     int_t streamId,
@@ -211,30 +212,32 @@ int dSchurCompUpdate_GPU(
     int_t mcb,
     int_t buffer_size, int_t lsub_len, int_t usub_len,
     int_t ldt, int_t k0,
-    dsluGPU_t *sluGPU, gridinfo_t *grid
+    dsluGPU_t *sluGPU, gridinfo_t *grid,
+    SuperLUStat_t *
 );
 
 
 extern void dCopyLUToGPU3D (int* isNodeInMyGrid, dLocalLU_t *A_host,
            dsluGPU_t *sluGPU, Glu_persist_t *Glu_persist, int_t n,
-	   gridinfo3d_t *grid3d, int_t buffer_size, int_t bigu_size, int_t ldt);
+	   gridinfo3d_t *grid3d, int_t buffer_size, int_t bigu_size, int_t ldt,
+    	   SuperLUStat_t *
+	   );
 
 extern int dreduceAllAncestors3d_GPU(int_t ilvl, int_t* myNodeCount,
                               int_t** treePerm,    dLUValSubBuf_t*LUvsb,
                               dLUstruct_t* LUstruct, gridinfo3d_t* grid3d,
                               dsluGPU_t *sluGPU,  d2Hreduce_t* d2Hred,
-                              factStat_t *factStat, HyP_t* HyP, SCT_t* SCT );
+                              factStat_t *factStat, HyP_t* HyP, SCT_t* SCT,
+    			      SuperLUStat_t *
+			      );
 
 extern void dsyncAllfunCallStreams(dsluGPU_t* sluGPU, SCT_t* SCT);
-extern int dfree_LUstruct_gpu (dLUstruct_gpu_t *A_gpu);
+extern int dfree_LUstruct_gpu (dsluGPU_t *sluGPU, SuperLUStat_t *);
 
 //int freeSluGPU(dsluGPU_t *sluGPU);
 
 extern void dPrint_matrix( char *desc, int_t m, int_t n, double *dA, int_t lda );
 
-/*to print out various statistics*/
-void dprintGPUStats(dLUstruct_gpu_t *A_gpu);
-
 #ifdef __cplusplus
 }
 #endif
