@@ -538,7 +538,7 @@ pdgssvx(superlu_dist_options_t *options, SuperMatrix *A,
     int_t   nnz_loc, nnz, iinfo;
     int     m_loc, fst_row, icol;
     int     colequ, Equil, factored, job, notran, rowequ, need_value;
-    int     i, j, irow, m, n, permc_spec;
+    int_t     i, j, irow, m, n, permc_spec;
     int     iam, iam_g;
     int     ldx;  /* LDA for matrix X (local). */
     char    equed[1], norm[1];
@@ -716,7 +716,6 @@ pdgssvx(superlu_dist_options_t *options, SuperMatrix *A,
 	} else { /* Compute R & C from scratch */
             /* Compute the row and column scalings. */
 	    pdgsequ(A, R, C, &rowcnd, &colcnd, &amax, &iinfo, grid);
-
 	    if ( iinfo > 0 ) {
 		if ( iinfo <= m ) {
 #if ( PRNTlevel>=1 )
