@@ -22,6 +22,9 @@ at the top-level directory.
 
 #include <math.h>
 #include "superlu_ddefs.h"
+#ifdef GPU_ACC
+#include "gpu_api_utils.h"
+#endif
 
 /*! \brief Gather A from the distributed compressed row format to global A in compressed column format.
  */
@@ -430,6 +433,53 @@ void dLUstructFree(dLUstruct_t *LUstruct)
 #endif
 }
 
+void
+dDestroy_Tree(int_t n, gridinfo_t *grid, dLUstruct_t *LUstruct)
+{
+    int_t i, nb, nsupers;
+    Glu_persist_t *Glu_persist = LUstruct->Glu_persist;
+    dLocalLU_t *Llu = LUstruct->Llu;
+#if ( DEBUGlevel>=1 )
+    int iam;
+    MPI_Comm_rank( MPI_COMM_WORLD, &iam );
+    CHECK_MALLOC(iam, "Enter dDestroy_Tree()");
+#endif
+
+    nsupers = Glu_persist->supno[n-1] + 1;
+
+	nb = CEILING(nsupers, grid->npcol);
+	for (i=0;i<nb;++i){
+        if(Llu->LBtree_ptr[i].empty_==NO){    
+			// BcTree_Destroy(Llu->LBtree_ptr[i],LUstruct->dt);
+            C_BcTree_Nullify(&Llu->LBtree_ptr[i]);
+		}
+        if(Llu->UBtree_ptr[i].empty_==NO){  
+			// BcTree_Destroy(Llu->UBtree_ptr[i],LUstruct->dt);
+            C_BcTree_Nullify(&Llu->UBtree_ptr[i]);
+		}		
+	}
+	SUPERLU_FREE(Llu->LBtree_ptr);
+	SUPERLU_FREE(Llu->UBtree_ptr);
+	
+ 	nb = CEILING(nsupers, grid->nprow);
+	for (i=0;i<nb;++i){
+        if(Llu->LRtree_ptr[i].empty_==NO){             
+			// RdTree_Destroy(Llu->LRtree_ptr[i],LUstruct->dt);
+            C_RdTree_Nullify(&Llu->LRtree_ptr[i]);
+		}
+        if(Llu->URtree_ptr[i].empty_==NO){ 
+			// RdTree_Destroy(Llu->URtree_ptr[i],LUstruct->dt);
+            C_RdTree_Nullify(&Llu->URtree_ptr[i]);
+		}		
+	}
+	SUPERLU_FREE(Llu->LRtree_ptr);
+	SUPERLU_FREE(Llu->URtree_ptr);
+
+#if ( DEBUGlevel>=1 )
+    CHECK_MALLOC(iam, "Exit dDestroy_Tree()");
+#endif
+}
+
 /*! \brief Destroy distributed L & U matrices. */
 void
 dDestroy_LU(int_t n, gridinfo_t *grid, dLUstruct_t *LUstruct)
@@ -449,26 +499,30 @@ dDestroy_LU(int_t n, gridinfo_t *grid, dLUstruct_t *LUstruct)
     nsupers = Glu_persist->supno[n-1] + 1;
 
     nb = CEILING(nsupers, grid->npcol);
-    for (i = 0; i < nb; ++i) 
-	if ( Llu->Lrowind_bc_ptr[i] ) {
-	    SUPERLU_FREE (Llu->Lrowind_bc_ptr[i]);
-#if 0 // Sherry: the following is not allocated with cudaHostAlloc    
-    //#ifdef GPU_ACC
-	    checkCuda(cudaFreeHost(Llu->Lnzval_bc_ptr[i]));
-#endif
-	    SUPERLU_FREE (Llu->Lnzval_bc_ptr[i]);
-	}
+    // for (i = 0; i < nb; ++i) 
+	// if ( Llu->Lrowind_bc_ptr[i] ) {
+	    // SUPERLU_FREE (Llu->Lrowind_bc_ptr[i]);
+	    // SUPERLU_FREE (Llu->Lnzval_bc_ptr[i]);
+	// }
     SUPERLU_FREE (Llu->Lrowind_bc_ptr);
+    SUPERLU_FREE (Llu->Lrowind_bc_dat);
+    SUPERLU_FREE (Llu->Lrowind_bc_offset);    
     SUPERLU_FREE (Llu->Lnzval_bc_ptr);
+    SUPERLU_FREE (Llu->Lnzval_bc_dat);
+    SUPERLU_FREE (Llu->Lnzval_bc_offset);
 
-    nb = CEILING(nsupers, grid->nprow);
-    for (i = 0; i < nb; ++i)
-	if ( Llu->Ufstnz_br_ptr[i] ) {
-	    SUPERLU_FREE (Llu->Ufstnz_br_ptr[i]);
-	    SUPERLU_FREE (Llu->Unzval_br_ptr[i]);
-	}
+    // nb = CEILING(nsupers, grid->nprow);
+    // for (i = 0; i < nb; ++i)
+	// if ( Llu->Ufstnz_br_ptr[i] ) {
+	//     SUPERLU_FREE (Llu->Ufstnz_br_ptr[i]);
+	//     SUPERLU_FREE (Llu->Unzval_br_ptr[i]);
+	// }
     SUPERLU_FREE (Llu->Ufstnz_br_ptr);
+    SUPERLU_FREE (Llu->Ufstnz_br_dat);
+    SUPERLU_FREE (Llu->Ufstnz_br_offset);
     SUPERLU_FREE (Llu->Unzval_br_ptr);
+    SUPERLU_FREE (Llu->Unzval_br_dat);
+    SUPERLU_FREE (Llu->Unzval_br_offset);
 
     /* The following can be freed after factorization. */
     SUPERLU_FREE(Llu->ToRecv);
@@ -486,39 +540,79 @@ dDestroy_LU(int_t n, gridinfo_t *grid, dLUstruct_t *LUstruct)
     SUPERLU_FREE(Llu->bsendx_plist);
     SUPERLU_FREE(Llu->mod_bit);
 
-    nb = CEILING(nsupers, grid->npcol);
-    for (i = 0; i < nb; ++i) 
-	if ( Llu->Lindval_loc_bc_ptr[i]!=NULL) {
-	    SUPERLU_FREE (Llu->Lindval_loc_bc_ptr[i]);
-	}	
+    // nb = CEILING(nsupers, grid->npcol);
+    // for (i = 0; i < nb; ++i) 
+	// if ( Llu->Lindval_loc_bc_ptr[i]!=NULL) {
+	//     SUPERLU_FREE (Llu->Lindval_loc_bc_ptr[i]);
+	// }	
     SUPERLU_FREE(Llu->Lindval_loc_bc_ptr);
+    SUPERLU_FREE(Llu->Lindval_loc_bc_dat);
+    SUPERLU_FREE(Llu->Lindval_loc_bc_offset);
 	
-    nb = CEILING(nsupers, grid->npcol);
-    for (i=0; i<nb; ++i) {
-	if(Llu->Linv_bc_ptr[i]!=NULL) {
-	    SUPERLU_FREE(Llu->Linv_bc_ptr[i]);
-	}
-	if(Llu->Uinv_bc_ptr[i]!=NULL){
-	    SUPERLU_FREE(Llu->Uinv_bc_ptr[i]);
-	}	
-    }
+    // nb = CEILING(nsupers, grid->npcol);
+    // for (i=0; i<nb; ++i) {
+	// // if(Llu->Linv_bc_ptr[i]!=NULL) {
+	// //     SUPERLU_FREE(Llu->Linv_bc_ptr[i]);
+	// // }
+
+	// if(Llu->Uinv_bc_ptr[i]!=NULL){
+	//     SUPERLU_FREE(Llu->Uinv_bc_ptr[i]);
+	// }	
+    // }
     SUPERLU_FREE(Llu->Linv_bc_ptr);
+    SUPERLU_FREE(Llu->Linv_bc_dat);
+    SUPERLU_FREE(Llu->Linv_bc_offset);
     SUPERLU_FREE(Llu->Uinv_bc_ptr);
+    SUPERLU_FREE(Llu->Uinv_bc_dat);
+    SUPERLU_FREE(Llu->Uinv_bc_offset);
     SUPERLU_FREE(Llu->Unnz);
 	
-    nb = CEILING(nsupers, grid->npcol);
-    for (i = 0; i < nb; ++i)
-	if ( Llu->Urbs[i] ) {
-	    SUPERLU_FREE(Llu->Ucb_indptr[i]);
-	    SUPERLU_FREE(Llu->Ucb_valptr[i]);
-	}
+    // nb = CEILING(nsupers, grid->npcol);
+    // for (i = 0; i < nb; ++i)
+	// if ( Llu->Urbs[i] ) {
+	//     SUPERLU_FREE(Llu->Ucb_indptr[i]);
+	//     SUPERLU_FREE(Llu->Ucb_valptr[i]);
+	// }
     SUPERLU_FREE(Llu->Ucb_indptr);
+    SUPERLU_FREE(Llu->Ucb_inddat);
+    SUPERLU_FREE(Llu->Ucb_indoffset);
+
     SUPERLU_FREE(Llu->Ucb_valptr);	
+    SUPERLU_FREE(Llu->Ucb_valdat);	
+    SUPERLU_FREE(Llu->Ucb_valoffset);	
     SUPERLU_FREE(Llu->Urbs);
 
     SUPERLU_FREE(Glu_persist->xsup);
     SUPERLU_FREE(Glu_persist->supno);
 
+#ifdef GPU_ACC
+	checkGPU (gpuFree (Llu->d_xsup));
+	checkGPU (gpuFree (Llu->d_LRtree_ptr));
+	checkGPU (gpuFree (Llu->d_LBtree_ptr));
+	checkGPU (gpuFree (Llu->d_URtree_ptr));
+	checkGPU (gpuFree (Llu->d_UBtree_ptr));    
+	checkGPU (gpuFree (Llu->d_ilsum));
+	checkGPU (gpuFree (Llu->d_Lrowind_bc_dat));
+	checkGPU (gpuFree (Llu->d_Lrowind_bc_offset));
+	checkGPU (gpuFree (Llu->d_Lnzval_bc_dat));
+	checkGPU (gpuFree (Llu->d_Lnzval_bc_offset));
+	checkGPU (gpuFree (Llu->d_Linv_bc_dat));
+	checkGPU (gpuFree (Llu->d_Uinv_bc_dat));
+	checkGPU (gpuFree (Llu->d_Linv_bc_offset));
+	checkGPU (gpuFree (Llu->d_Uinv_bc_offset));
+	checkGPU (gpuFree (Llu->d_Lindval_loc_bc_dat));
+	checkGPU (gpuFree (Llu->d_Lindval_loc_bc_offset));
+
+    checkGPU (gpuFree (Llu->d_Ucolind_bc_dat));    
+    checkGPU (gpuFree (Llu->d_Ucolind_bc_offset));    
+    checkGPU (gpuFree (Llu->d_Unzval_bc_dat));    
+    checkGPU (gpuFree (Llu->d_Unzval_bc_offset));    
+    checkGPU (gpuFree (Llu->d_Uindval_loc_bc_dat));    
+    checkGPU (gpuFree (Llu->d_Uindval_loc_bc_offset));    
+
+#endif
+
+
 #if ( DEBUGlevel>=1 )
     CHECK_MALLOC(iam, "Exit dDestroy_LU()");
 #endif
@@ -849,48 +943,6 @@ void pdinf_norm_error(int iam, int_t n, int_t nrhs, double x[], int_t ldx,
     }
 }
 
-/*! \brief Destroy broadcast and reduction trees used in triangular solve */
-void
-dDestroy_Tree(int_t n, gridinfo_t *grid, dLUstruct_t *LUstruct)
-{
-    int_t i, nb, nsupers;
-    Glu_persist_t *Glu_persist = LUstruct->Glu_persist;
-    dLocalLU_t *Llu = LUstruct->Llu;
-#if ( DEBUGlevel>=1 )
-    int iam;
-    MPI_Comm_rank( MPI_COMM_WORLD, &iam );
-    CHECK_MALLOC(iam, "Enter Destroy_Tree()");
-#endif
-
-    nsupers = Glu_persist->supno[n-1] + 1;
-
-    nb = CEILING(nsupers, grid->npcol);
-    for (i=0;i<nb;++i){
-	if(Llu->LBtree_ptr[i]!=NULL){
-		BcTree_Destroy(Llu->LBtree_ptr[i],LUstruct->dt);
-	}
-	if(Llu->UBtree_ptr[i]!=NULL){
-		BcTree_Destroy(Llu->UBtree_ptr[i],LUstruct->dt);
-	}		
-    }
-    SUPERLU_FREE(Llu->LBtree_ptr);
-    SUPERLU_FREE(Llu->UBtree_ptr);
-	
-    nb = CEILING(nsupers, grid->nprow);
-    for (i=0;i<nb;++i){
-	if(Llu->LRtree_ptr[i]!=NULL){
-		RdTree_Destroy(Llu->LRtree_ptr[i],LUstruct->dt);
-	}
-	if(Llu->URtree_ptr[i]!=NULL){
-		RdTree_Destroy(Llu->URtree_ptr[i],LUstruct->dt);
-	}		
-    }
-    SUPERLU_FREE(Llu->LRtree_ptr);
-    SUPERLU_FREE(Llu->URtree_ptr);
-
-#if ( DEBUGlevel>=1 )
-    CHECK_MALLOC(iam, "Exit dDestroy_Tree()");
-#endif
-}
+/*! \brief Destroy distributed L & U matrices. */
 
 
