@@ -784,6 +784,10 @@ psCompute_Diag_Inv(int_t n, sLUstruct_t *LUstruct,gridinfo_t *grid,
  * Arguments
  * =========
  *
+ * options (input) superlu_dist_options_t*
+ *         The structure defines the input parameters to control
+ *         how the LU decomposition and triangular solve are performed.
+ *
  * n      (input) int (global)
  *        The order of the system of linear equations.
  *
@@ -837,7 +841,8 @@ psCompute_Diag_Inv(int_t n, sLUstruct_t *LUstruct,gridinfo_t *grid,
  */
 
 void
-psgstrs(int_t n, sLUstruct_t *LUstruct,
+psgstrs(superlu_dist_options_t *options, int_t n,
+        sLUstruct_t *LUstruct,
 	sScalePermstruct_t *ScalePermstruct,
 	gridinfo_t *grid, float *B,
 	int_t m_loc, int_t fst_row, int_t ldb, int nrhs,
@@ -865,10 +870,10 @@ psgstrs(int_t n, sLUstruct_t *LUstruct,
     int_t nroot_send, nroot_send_tmp;
     int_t  **Ufstnz_br_ptr = Llu->Ufstnz_br_ptr;
         /*-- Data structures used for broadcast and reduction trees. --*/
-    BcTree  *LBtree_ptr = Llu->LBtree_ptr;
-    RdTree  *LRtree_ptr = Llu->LRtree_ptr;
-    BcTree  *UBtree_ptr = Llu->UBtree_ptr;
-    RdTree  *URtree_ptr = Llu->URtree_ptr;
+    C_Tree  *LBtree_ptr = Llu->LBtree_ptr;
+    C_Tree  *LRtree_ptr = Llu->LRtree_ptr;
+    C_Tree  *UBtree_ptr = Llu->UBtree_ptr;
+    C_Tree  *URtree_ptr = Llu->URtree_ptr;
     int_t  *Urbs1; /* Number of row blocks in each block column of U. */
     int_t  *Urbs = Llu->Urbs; /* Number of row blocks in each block column of U. */
     Ucb_indptr_t **Ucb_indptr = Llu->Ucb_indptr;/* Vertical linked list pointing to Uindex[] */
@@ -894,30 +899,30 @@ psgstrs(int_t n, sLUstruct_t *LUstruct,
 
     double tmax;
     	/*-- Counts used for L-solve --*/
-    int_t  *fmod;         /* Modification count for L-solve --
+    int  *fmod;         /* Modification count for L-solve --
     			 Count the number of local block products to
     			 be summed into lsum[lk]. */
-    int_t fmod_tmp;
-    int_t  **fsendx_plist = Llu->fsendx_plist;
-    int_t  nfrecvx = Llu->nfrecvx; /* Number of X components to be recv'd. */
-    int_t  nfrecvx_buf=0;
-    int_t  *frecv;        /* Count of lsum[lk] contributions to be received
+    int  fmod_tmp;
+    int  **fsendx_plist = Llu->fsendx_plist;
+    int  nfrecvx = Llu->nfrecvx; /* Number of X components to be recv'd. */
+    int  nfrecvx_buf=0;
+    int  *frecv;        /* Count of lsum[lk] contributions to be received
     			     from processes in this row.
     			     It is only valid on the diagonal processes. */
-    int_t  frecv_tmp;
-    int_t  nfrecvmod = 0; /* Count of total modifications to be recv'd. */
-    int_t  nfrecv = 0; /* Count of total messages to be recv'd. */
-    int_t  nbrecv = 0; /* Count of total messages to be recv'd. */
-    int_t  nleaf = 0, nroot = 0;
-    int_t  nleaftmp = 0, nroottmp = 0;
+    int  frecv_tmp;
+    int  nfrecvmod = 0; /* Count of total modifications to be recv'd. */
+    int  nfrecv = 0; /* Count of total messages to be recv'd. */
+    int  nbrecv = 0; /* Count of total messages to be recv'd. */
+    int  nleaf = 0, nroot = 0;
+    int  nleaftmp = 0, nroottmp = 0;
     int_t  msgsize;
         /*-- Counts used for U-solve --*/
-    int_t  *bmod;         /* Modification count for U-solve. */
-    int_t  bmod_tmp;
-    int_t  **bsendx_plist = Llu->bsendx_plist;
-    int_t  nbrecvx = Llu->nbrecvx; /* Number of X components to be recv'd. */
-    int_t  nbrecvx_buf=0;
-    int_t  *brecv;        /* Count of modifications to be recv'd from
+    int  *bmod;         /* Modification count for U-solve. */
+    int  bmod_tmp;
+    int  **bsendx_plist = Llu->bsendx_plist;
+    int  nbrecvx = Llu->nbrecvx; /* Number of X components to be recv'd. */
+    int  nbrecvx_buf=0;
+    int  *brecv;        /* Count of modifications to be recv'd from
     			     processes in this row. */
     int_t  nbrecvmod = 0; /* Count of total modifications to be recv'd. */
     int_t flagx,flaglsum,flag;
@@ -930,7 +935,7 @@ psgstrs(int_t n, sLUstruct_t *LUstruct,
 
     int_t gik,iklrow,fnz;
 
-    int_t *mod_bit = Llu->mod_bit; /* flag contribution from each row block */
+    int *mod_bit = Llu->mod_bit; /* flag contribution from each row block */
     int INFO, pad;
     int_t tmpresult;
 
@@ -958,11 +963,11 @@ psgstrs(int_t n, sLUstruct_t *LUstruct,
     int thread_id = 0;
     yes_no_t empty;
     int_t sizelsum,sizertemp,aln_d,aln_i;
-    aln_d = ceil(CACHELINE/(double)dword);
-    aln_i = ceil(CACHELINE/(double)iword);
+    aln_d = 1; //ceil(CACHELINE/(double)dword);
+    aln_i = 1; //ceil(CACHELINE/(double)iword);
     int num_thread = 1;
 
-    maxsuper = sp_ienv_dist(3);
+    maxsuper = sp_ienv_dist(3, options);
 
 //#ifdef _OPENMP
 //#pragma omp threadprivate(thread_id)
@@ -1029,11 +1034,11 @@ psgstrs(int_t n, sLUstruct_t *LUstruct,
     Llu->SolveMsgSent = 0;
 
     /* Save the count to be altered so it can be used by
-       subsequent call to PDGSTRS. */
-    if ( !(fmod = intMalloc_dist(nlb*aln_i)) )
+       subsequent call to PSGSTRS. */
+    if ( !(fmod = int32Malloc_dist(nlb*aln_i)) )
 	ABORT("Malloc fails for fmod[].");
     for (i = 0; i < nlb; ++i) fmod[i*aln_i] = Llu->fmod[i];
-    if ( !(frecv = intCalloc_dist(nlb)) )
+    if ( !(frecv = int32Calloc_dist(nlb)) )
 	ABORT("Calloc fails for frecv[].");
     Llu->frecv = frecv;
 
@@ -1056,7 +1061,7 @@ psgstrs(int_t n, sLUstruct_t *LUstruct,
     ldalsum = Llu->ldalsum;
 
     /* Allocate working storage. */
-    knsupc = sp_ienv_dist(3);
+    knsupc = sp_ienv_dist(3, options);
     maxrecvsz = knsupc * nrhs + SUPERLU_MAX( XK_H, LSUM_H );
     sizelsum = (((size_t)ldalsum)*nrhs + nlb*LSUM_H);
     sizelsum = ((sizelsum + (aln_d - 1)) / aln_d) * aln_d;
@@ -1140,13 +1145,15 @@ psgstrs(int_t n, sLUstruct_t *LUstruct,
 
 	nbtree = 0;
 	for (lk=0;lk<nsupers_j;++lk){
-		if(LBtree_ptr[lk]!=NULL){
+		if(LBtree_ptr[lk].empty_==NO){
 			// printf("LBtree_ptr lk %5d\n",lk);
-			if(BcTree_IsRoot(LBtree_ptr[lk],'s')==NO){
+			//if(BcTree_IsRoot(LBtree_ptr[lk],'s')==NO){
+			if(C_BcTree_IsRoot(&LBtree_ptr[lk])==NO){
 				nbtree++;
-				if(BcTree_getDestCount(LBtree_ptr[lk],'s')>0)nfrecvx_buf++;
+				//if(BcTree_getDestCount(LBtree_ptr[lk],'s')>0)nfrecvx_buf++;
+				if(LBtree_ptr[lk].destCnt_>0)nfrecvx_buf++;
 			}
-			BcTree_allocateRequest(LBtree_ptr[lk],'s');
+			//BcTree_allocateRequest(LBtree_ptr[lk],'s');
 		}
 	}
 
@@ -1158,8 +1165,6 @@ psgstrs(int_t n, sLUstruct_t *LUstruct,
 	nleaf=0;
 	nfrecvmod=0;
 
-
-
 if(procs==1){
 	for (lk=0;lk<nsupers_i;++lk){
 		gb = myrow+lk*grid->nprow;  /* not sure */
@@ -1172,10 +1177,11 @@ if(procs==1){
 	}
 }else{
 	for (lk=0;lk<nsupers_i;++lk){
-		if(LRtree_ptr[lk]!=NULL){
+		if(LRtree_ptr[lk].empty_==NO){
 			nrtree++;
-			RdTree_allocateRequest(LRtree_ptr[lk],'s');
-			frecv[lk] = RdTree_GetDestCount(LRtree_ptr[lk],'s');
+			//RdTree_allocateRequest(LRtree_ptr[lk],'s');
+			//frecv[lk] = RdTree_GetDestCount(LRtree_ptr[lk],'s');
+			frecv[lk] = LRtree_ptr[lk].destCnt_;
 			nfrecvmod += frecv[lk];
 		}else{
 			gb = myrow+lk*grid->nprow;  /* not sure */
@@ -1192,7 +1198,6 @@ if(procs==1){
 	}
 }
 
-
 #ifdef _OPENMP
 #pragma omp simd
 #endif
@@ -1204,8 +1209,6 @@ if(procs==1){
 
 	log_memory(nlb*aln_i*iword+nlb*iword+(CEILING( nsupers, Pr )+CEILING( nsupers, Pc ))*aln_i*2.0*iword+ nsupers_i*iword + sizelsum*num_thread * dword + (ldalsum * nrhs + nlb * XK_H) *dword + (sizertemp*num_thread + 1)*dword+maxrecvsz*(nfrecvx+1)*dword, stat);	//account for fmod, frecv, leaf_send, root_send, leafsups, recvbuf_BC_fwd	, lsum, x, rtemp
 
-
-
 #if ( DEBUGlevel>=2 )
 	printf("(%2d) nfrecvx %4d,  nfrecvmod %4d,  nleaf %4d\n,  nbtree %4d\n,  nrtree %4d\n",
 			iam, nfrecvx, nfrecvmod, nleaf, nbtree, nrtree);
@@ -1238,7 +1241,6 @@ if(procs==1){
 	fflush(stdout);
 #endif
 
-
 #ifdef _OPENMP
 #pragma omp parallel default (shared)
 	{
@@ -1291,9 +1293,6 @@ if(procs==1){
 					&knsupc, &beta, rtemp_loc, &knsupc );
 #endif
 
-#ifdef _OPENMP
-#pragma omp simd
-#endif
 			for (i=0 ; i<knsupc*nrhs ; i++){
 				x[ii+i] = rtemp_loc[i];
 			}
@@ -1319,7 +1318,7 @@ if(procs==1){
 			 * Send Xk to process column Pc[k].
 			 */
 
-			if(LBtree_ptr[lk]!=NULL){
+			if(LBtree_ptr[lk].empty_==NO){
 				lib = LBi( k, grid ); /* Local block number, row-wise. */
 				ii = X_BLK( lib );
 
@@ -1383,7 +1382,7 @@ if(procs==1){
 		     * Send Xk to process column Pc[k].
 		     */
 
-		    if (LBtree_ptr[lk]!=NULL) {
+		    if (LBtree_ptr[lk].empty_==NO) {
 			lib = LBi( k, grid ); /* Local block number, row-wise. */
 			ii = X_BLK( lib );
 
@@ -1442,11 +1441,15 @@ if(procs==1){
 			gb = mycol+lk*grid->npcol;  /* not sure */
 			lib = LBi( gb, grid ); /* Local block number, row-wise. */
 			ii = X_BLK( lib );
-			BcTree_forwardMessageSimple(LBtree_ptr[lk],&x[ii - XK_H],BcTree_GetMsgSize(LBtree_ptr[lk],'s')*nrhs+XK_H,'s');
+			//BcTree_forwardMessageSimple(LBtree_ptr[lk],&x[ii - XK_H],BcTree_GetMsgSize(LBtree_ptr[lk],'s')*nrhs+XK_H,'s');
+			C_BcTree_forwardMessageSimple(&LBtree_ptr[lk], &x[ii - XK_H], LBtree_ptr[lk].msgSize_*nrhs+XK_H);
+			
 		}else{ // this is a reduce forwarding
 			lk = -lk - 1;
 			il = LSUM_BLK( lk );
-			RdTree_forwardMessageSimple(LRtree_ptr[lk],&lsum[il - LSUM_H ],RdTree_GetMsgSize(LRtree_ptr[lk],'s')*nrhs+LSUM_H,'s');
+			//RdTree_forwardMessageSimple(LRtree_ptr[lk],&lsum[il - LSUM_H ],RdTree_GetMsgSize(LRtree_ptr[lk],'s')*nrhs+LSUM_H,'s');
+			C_RdTree_forwardMessageSimple(&LRtree_ptr[lk],&lsum[il - LSUM_H],LRtree_ptr[lk].msgSize_*nrhs+LSUM_H);
+			
 		}
 	}
 
@@ -1513,9 +1516,10 @@ if(procs==1){
 				{
 				lk = LBj( k, grid );    /* local block number */
 
-				if(BcTree_getDestCount(LBtree_ptr[lk],'s')>0){
+				if(LBtree_ptr[lk].destCnt_>0){
 
-					BcTree_forwardMessageSimple(LBtree_ptr[lk],recvbuf0,BcTree_GetMsgSize(LBtree_ptr[lk],'s')*nrhs+XK_H,'s');
+					//BcTree_forwardMessageSimple(LBtree_ptr[lk],recvbuf0,BcTree_GetMsgSize(LBtree_ptr[lk],'s')*nrhs+XK_H,'s');
+					C_BcTree_forwardMessageSimple(&LBtree_ptr[lk], recvbuf0, LBtree_ptr[lk].msgSize_*nrhs+XK_H);
 					// nfrecvx_buf++;
 				}
 
@@ -1564,13 +1568,10 @@ if(procs==1){
 				thread_id = 0;
 				rtemp_loc = &rtemp[sizertemp* thread_id];
 				if ( fmod_tmp==0 ) {
-				    if(RdTree_IsRoot(LRtree_ptr[lk],'s')==YES){
+				    if(C_RdTree_IsRoot(&LRtree_ptr[lk])==YES){
 				    // ii = X_BLK( lk );
 					knsupc = SuperSize( k );
 					for (ii=1;ii<num_thread;ii++)
-				#ifdef _OPENMP
-				#pragma omp simd
-				#endif
 					    for (jj=0;jj<knsupc*nrhs;jj++)
 						lsum[il + jj ] += lsum[il + jj + ii*sizelsum];
 					ii = X_BLK( lk );
@@ -1638,8 +1639,10 @@ if(procs==1){
 					/*
 					 * Send Xk to process column Pc[k].
 					 */
-					if(LBtree_ptr[lk]!=NULL){
-					    BcTree_forwardMessageSimple(LBtree_ptr[lk],&x[ii - XK_H],BcTree_GetMsgSize(LBtree_ptr[lk],'s')*nrhs+XK_H,'s');
+					if(LBtree_ptr[lk].empty_==NO){
+					    //BcTree_forwardMessageSimple(LBtree_ptr[lk],&x[ii - XK_H],BcTree_GetMsgSize(LBtree_ptr[lk],'s')*nrhs+XK_H,'s');
+					    C_BcTree_forwardMessageSimple(&LBtree_ptr[lk], &x[ii - XK_H], LBtree_ptr[lk].msgSize_*nrhs+XK_H);
+					    
 					}
 					/*
 					 * Perform local block modifications.
@@ -1663,22 +1666,20 @@ if(procs==1){
 					il = LSUM_BLK( lk );
 					knsupc = SuperSize( k );
 					for (ii=1;ii<num_thread;ii++)
-					#ifdef _OPENMP
-					#pragma omp simd
-					#endif
 					    for (jj=0;jj<knsupc*nrhs;jj++)
 						lsum[il + jj] += lsum[il + jj + ii*sizelsum];
-					RdTree_forwardMessageSimple(LRtree_ptr[lk],&lsum[il-LSUM_H],RdTree_GetMsgSize(LRtree_ptr[lk],'s')*nrhs+LSUM_H,'s');
+					//RdTree_forwardMessageSimple(LRtree_ptr[lk],&lsum[il-LSUM_H],RdTree_GetMsgSize(LRtree_ptr[lk],'s')*nrhs+LSUM_H,'s');
+					C_RdTree_forwardMessageSimple(&LRtree_ptr[lk],&lsum[il - LSUM_H ],LRtree_ptr[lk].msgSize_*nrhs+LSUM_H);
 				    } /* end if-else */
 
 				    }
 
 				} /* end else MPI_TAG==RD_L */
-			        } /* check Tag */
+			    } /* check Tag */
 			} /* end for nfrecv ... */
                     } /* while not finished ... */
        	    }
-        } // Sherry: add back
+        } // end of parallel
 
 #if ( PRNTlevel>=2 )
 	t = SuperLU_timer_() - t;
@@ -1723,17 +1724,18 @@ if(procs==1){
 	log_memory(-nlb*aln_i*iword-nlb*iword-(CEILING( nsupers, Pr )+CEILING( nsupers, Pc ))*aln_i*iword- nsupers_i*iword -maxrecvsz*(nfrecvx+1)*dword, stat);	//account for fmod, frecv, leaf_send, leafsups, recvbuf_BC_fwd
 
 	for (lk=0;lk<nsupers_j;++lk){
-		if(LBtree_ptr[lk]!=NULL){
+		if(LBtree_ptr[lk].empty_==NO){
 			// if(BcTree_IsRoot(LBtree_ptr[lk],'s')==YES){
-			BcTree_waitSendRequest(LBtree_ptr[lk],'s');
+			//BcTree_waitSendRequest(LBtree_ptr[lk],'s');
+			C_BcTree_waitSendRequest(&LBtree_ptr[lk]);
 			// }
 			// deallocate requests here
 		}
 	}
 
 	for (lk=0;lk<nsupers_i;++lk){
-		if(LRtree_ptr[lk]!=NULL){
-			RdTree_waitSendRequest(LRtree_ptr[lk],'s');
+		if(LRtree_ptr[lk].empty_==NO){
+			C_RdTree_waitSendRequest(&LRtree_ptr[lk]);
 			// deallocate requests here
 		}
 	}
@@ -1753,11 +1755,11 @@ if(procs==1){
 	 *---------------------------------------------------*/
 
 	/* Save the count to be altered so it can be used by
-	   subsequent call to PDGSTRS. */
-	if ( !(bmod = intMalloc_dist(nlb*aln_i)) )
+	   subsequent call to PSGSTRS. */
+	if ( !(bmod = int32Malloc_dist(nlb*aln_i)) )
 		ABORT("Malloc fails for bmod[].");
 	for (i = 0; i < nlb; ++i) bmod[i*aln_i] = Llu->bmod[i];
-	if ( !(brecv = intCalloc_dist(nlb)) )
+	if ( !(brecv = int32Calloc_dist(nlb)) )
 		ABORT("Calloc fails for brecv[].");
 	Llu->brecv = brecv;
 
@@ -1803,12 +1805,13 @@ if(procs==1){
 #endif
 
 #if ( DEBUGlevel>=2 )
-	for (p = 0; p < Pr*Pc; ++p) {
+        nub = CEILING( nsupers, Pc ); /* Number of local block columns. */
+        for (p = 0; p < Pr*Pc; ++p) {
 	    if (iam == p) {
-		printf("(%2d) .. Ublocks %d\n", iam, Ublocks);
+		printf("(%2d) .. Ublocks %d, nub %d\n",iam,Ublocks,nub); fflush(stdout);
 		for (lb = 0; lb < nub; ++lb) {
 		    printf("(%2d) Local col %2d: # row blocks %2d\n",
-				iam, lb, Urbs[lb]);
+				iam, lb, Urbs[lb]); fflush(stdout);
 		    if ( Urbs[lb] ) {
 			for (i = 0; i < Urbs[lb]; ++i)
 			    printf("(%2d) .. row blk %2d:\
@@ -1843,13 +1846,13 @@ if(procs==1){
 
 	nbtree = 0;
 	for (lk=0;lk<nsupers_j;++lk){
-	    if(UBtree_ptr[lk]!=NULL){
+	    if(UBtree_ptr[lk].empty_==NO){
 		// printf("UBtree_ptr lk %5d\n",lk);
-		if(BcTree_IsRoot(UBtree_ptr[lk],'s')==NO){
+		if(C_BcTree_IsRoot(&UBtree_ptr[lk])==NO){
 			nbtree++;
-			if(BcTree_getDestCount(UBtree_ptr[lk],'s')>0)nbrecvx_buf++;
+			if(UBtree_ptr[lk].destCnt_>0)nbrecvx_buf++;
 		}
-		BcTree_allocateRequest(UBtree_ptr[lk],'s');
+		//BcTree_allocateRequest(UBtree_ptr[lk],'s');
 	    }
 	}
 
@@ -1860,12 +1863,12 @@ if(procs==1){
 	nrtree = 0;
 	nroot=0;
 	for (lk=0;lk<nsupers_i;++lk){
-		if(URtree_ptr[lk]!=NULL){
+		if(URtree_ptr[lk].empty_==NO){
 			// printf("here lk %5d myid %5d\n",lk,iam);
 			// fflush(stdout);
 			nrtree++;
-			RdTree_allocateRequest(URtree_ptr[lk],'s');
-			brecv[lk] = RdTree_GetDestCount(URtree_ptr[lk],'s');
+			//RdTree_allocateRequest(URtree_ptr[lk],'s');
+			brecv[lk] = URtree_ptr[lk].destCnt_;
 			nbrecvmod += brecv[lk];
 		}else{
 			gb = myrow+lk*grid->nprow;  /* not sure */
@@ -1881,9 +1884,6 @@ if(procs==1){
 		}
 	}
 
-#ifdef _OPENMP
-#pragma omp simd
-#endif
 	for (i = 0; i < nlb; ++i) bmod[i*aln_i] += brecv[i];
 	// for (i = 0; i < nlb; ++i)printf("bmod[i]: %5d\n",bmod[i]);
 
@@ -1965,9 +1965,6 @@ if(procs==1){
 						&alpha, Uinv, &knsupc, &x[ii],
 						&knsupc, &beta, rtemp_loc, &knsupc );
 #endif
-				#ifdef _OPENMP
-					#pragma omp simd
-				#endif
 				for (i=0 ; i<knsupc*nrhs ; i++){
 					x[ii+i] = rtemp_loc[i];
 				}
@@ -1998,7 +1995,7 @@ if(procs==1){
 			 * Send Xk to process column Pc[k].
 			 */
 
-			if(UBtree_ptr[lk]!=NULL){
+			if(UBtree_ptr[lk].empty_==NO){
 #ifdef _OPENMP
 #pragma omp atomic capture
 #endif
@@ -2051,11 +2048,13 @@ for (i=0;i<nroot_send;i++){
 		gb = mycol+lk*grid->npcol;  /* not sure */
 		lib = LBi( gb, grid ); /* Local block number, row-wise. */
 		ii = X_BLK( lib );
-		BcTree_forwardMessageSimple(UBtree_ptr[lk],&x[ii - XK_H],BcTree_GetMsgSize(UBtree_ptr[lk],'s')*nrhs+XK_H,'s');
+		//BcTree_forwardMessageSimple(UBtree_ptr[lk],&x[ii - XK_H],BcTree_GetMsgSize(UBtree_ptr[lk],'s')*nrhs+XK_H,'s');
+		C_BcTree_forwardMessageSimple(&UBtree_ptr[lk], &x[ii - XK_H], UBtree_ptr[lk].msgSize_*nrhs+XK_H);
 	}else{ // this is a reduce forwarding
 		lk = -lk - 1;
 		il = LSUM_BLK( lk );
-		RdTree_forwardMessageSimple(URtree_ptr[lk],&lsum[il - LSUM_H ],RdTree_GetMsgSize(URtree_ptr[lk],'s')*nrhs+LSUM_H,'s');
+		//RdTree_forwardMessageSimple(URtree_ptr[lk],&lsum[il - LSUM_H ],RdTree_GetMsgSize(URtree_ptr[lk],'s')*nrhs+LSUM_H,'s');
+		C_RdTree_forwardMessageSimple(&URtree_ptr[lk],&lsum[il - LSUM_H ],URtree_ptr[lk].msgSize_*nrhs+LSUM_H);
 	}
 }
 
@@ -2105,9 +2104,10 @@ for (i=0;i<nroot_send;i++){
 		    // --nfrecvx;
 		    nbrecvx_buf++;
 		    lk = LBj( k, grid );    /* local block number */
-		    if(BcTree_getDestCount(UBtree_ptr[lk],'s')>0){
+		    if(UBtree_ptr[lk].destCnt_>0){
 
-			BcTree_forwardMessageSimple(UBtree_ptr[lk],recvbuf0,BcTree_GetMsgSize(UBtree_ptr[lk],'s')*nrhs+XK_H,'s');
+			// BcTree_forwardMessageSimple(UBtree_ptr[lk],recvbuf0,BcTree_GetMsgSize(UBtree_ptr[lk],'d')*nrhs+XK_H,'d');
+			C_BcTree_forwardMessageSimple(&UBtree_ptr[lk], recvbuf0, UBtree_ptr[lk].msgSize_*nrhs+XK_H);
 			// nfrecvx_buf++;
 		    }
 
@@ -2127,9 +2127,6 @@ for (i=0;i<nroot_send;i++){
 		    tempv = &recvbuf0[LSUM_H];
 		    il = LSUM_BLK( lk );
 		    RHS_ITERATE(j) {
-		#ifdef _OPENMP
-		#pragma omp simd
-		#endif
 			for (i = 0; i < knsupc; ++i)
 			    lsum[i + il + j*knsupc + thread_id*sizelsum] += tempv[i + j*knsupc];
 		    }
@@ -2140,21 +2137,16 @@ for (i=0;i<nroot_send;i++){
 		    thread_id = 0;
 		    rtemp_loc = &rtemp[sizertemp* thread_id];
 		    if ( bmod_tmp==0 ) {
-			if(RdTree_IsRoot(URtree_ptr[lk],'s')==YES){
+			//if(RdTree_IsRoot(URtree_ptr[lk],'s')==YES){
+			if(C_RdTree_IsRoot(&URtree_ptr[lk])==YES){
 
 			    knsupc = SuperSize( k );
 			    for (ii=1;ii<num_thread;ii++)
-			#ifdef _OPENMP
-			#pragma omp simd
-			#endif
 				for (jj=0;jj<knsupc*nrhs;jj++)
 					lsum[il+ jj ] += lsum[il + jj + ii*sizelsum];
 
 			    ii = X_BLK( lk );
 			    RHS_ITERATE(j)
-			#ifdef _OPENMP
-			#pragma omp simd
-			#endif
 				for (i = 0; i < knsupc; ++i)
 				    x[i + ii + j*knsupc] += lsum[i + il + j*knsupc ];
 
@@ -2212,8 +2204,9 @@ for (i=0;i<nroot_send;i++){
 			    /*
 			     * Send Xk to process column Pc[k].
 			     */
-			    if(UBtree_ptr[lk]!=NULL){
-				BcTree_forwardMessageSimple(UBtree_ptr[lk],&x[ii - XK_H],BcTree_GetMsgSize(UBtree_ptr[lk],'s')*nrhs+XK_H,'s');
+			    if(UBtree_ptr[lk].empty_==NO){
+				//BcTree_forwardMessageSimple(UBtree_ptr[lk],&x[ii - XK_H],BcTree_GetMsgSize(UBtree_ptr[lk],'s')*nrhs+XK_H,'s');
+				C_BcTree_forwardMessageSimple(&UBtree_ptr[lk], &x[ii - XK_H], UBtree_ptr[lk].msgSize_*nrhs+XK_H);
 			    }
 
 			    /*
@@ -2230,13 +2223,11 @@ for (i=0;i<nroot_send;i++){
 				knsupc = SuperSize( k );
 
 				for (ii=1;ii<num_thread;ii++)
-			#ifdef _OPENMP
-			#pragma omp simd
-			#endif
 				    for (jj=0;jj<knsupc*nrhs;jj++)
 					lsum[il+ jj ] += lsum[il + jj + ii*sizelsum];
 
-				RdTree_forwardMessageSimple(URtree_ptr[lk],&lsum[il-LSUM_H],RdTree_GetMsgSize(URtree_ptr[lk],'s')*nrhs+LSUM_H,'s');
+				//RdTree_forwardMessageSimple(URtree_ptr[lk],&lsum[il-LSUM_H],RdTree_GetMsgSize(URtree_ptr[lk],'s')*nrhs+LSUM_H,'s');
+				C_RdTree_forwardMessageSimple(&URtree_ptr[lk],&lsum[il - LSUM_H ],URtree_ptr[lk].msgSize_*nrhs+LSUM_H);
 			    } /* end if-else */
 			} /* end if bmod_tmp==0 */
 		    } /* end if MPI_TAG==RD_U */
@@ -2301,7 +2292,7 @@ for (i=0;i<nroot_send;i++){
 		tmp3 = SUPERLU_MAX(tmp3,stat_loc[i]->utime[SOL_COMM]);
 		tmp4 += stat_loc[i]->ops[SOLVE];
 #if ( PRNTlevel>=2 )
-		f(iam==0)printf("thread %5d gemm %9.5f\n",i,stat_loc[i]->utime[SOL_GEMM]);
+		if(iam==0)printf("thread %5d gemm %9.5f\n",i,stat_loc[i]->utime[SOL_GEMM]);
 #endif
 	}
 
@@ -2330,17 +2321,17 @@ for (i=0;i<nroot_send;i++){
 	log_memory(-nlb*aln_i*iword-nlb*iword - nsupers_i*iword - (CEILING( nsupers, Pr )+CEILING( nsupers, Pc ))*aln_i*iword - maxrecvsz*(nbrecvx+1)*dword - sizelsum*num_thread * dword - (ldalsum * nrhs + nlb * XK_H) *dword - (sizertemp*num_thread + 1)*dword, stat);	//account for bmod, brecv, root_send, rootsups, recvbuf_BC_fwd,rtemp,lsum,x
 
 	for (lk=0;lk<nsupers_j;++lk){
-		if(UBtree_ptr[lk]!=NULL){
+		if(UBtree_ptr[lk].empty_==NO){
 			// if(BcTree_IsRoot(LBtree_ptr[lk],'s')==YES){
-			BcTree_waitSendRequest(UBtree_ptr[lk],'s');
+			C_BcTree_waitSendRequest(&UBtree_ptr[lk]);
 			// }
 			// deallocate requests here
 		}
 	}
 
 	for (lk=0;lk<nsupers_i;++lk){
-		if(URtree_ptr[lk]!=NULL){
-			RdTree_waitSendRequest(URtree_ptr[lk],'s');
+		if(URtree_ptr[lk].empty_==NO){
+			C_RdTree_waitSendRequest(&URtree_ptr[lk]);
 			// deallocate requests here
 		}
 	}
@@ -2359,7 +2350,7 @@ for (i=0;i<nroot_send;i++){
 		MPI_Reduce (&msg_vol, &msg_vol_max,
 				1, MPI_FLOAT, MPI_MAX, 0, grid->comm);
 		if (!iam) {
-			printf ("\tPDGSTRS comm stat:"
+			printf ("\tPSGSTRS comm stat:"
 				"\tAvg\tMax\t\tAvg\tMax\n"
 				"\t\t\tCount:\t%.0f\t%.0f\tVol(MB)\t%.2f\t%.2f\n",
 				msg_cnt_sum / Pr / Pc, msg_cnt_max,
