@@ -63,6 +63,10 @@ _fcd ftcs3;
  * Arguments
  * =========
  *
+ * options (input) superlu_dist_options_t*
+ *         The structure defines the input parameters to control
+ *         how the LU decomposition and triangular solve are performed.
+ *
  * n      (input) int (global)
  *        The order of the system of linear equations.
  *
@@ -94,7 +98,8 @@ _fcd ftcs3;
  * </pre>
  */
 
-void psgstrs1(int_t n, sLUstruct_t *LUstruct, gridinfo_t *grid,
+void psgstrs1(superlu_dist_options_t *options, int_t n,
+              sLUstruct_t *LUstruct, gridinfo_t *grid,
 	      float *x, int nrhs, SuperLUStat_t *stat, int *info)
 {
     Glu_persist_t *Glu_persist = LUstruct->Glu_persist;
@@ -109,7 +114,8 @@ void psgstrs1(int_t n, sLUstruct_t *LUstruct, gridinfo_t *grid,
     Ucb_indptr_t **Ucb_indptr;/* Vertical linked list pointing to Uindex[] */
     int_t  **Ucb_valptr;      /* Vertical linked list pointing to Unzval[] */
     int    iam, kcol, krow, mycol, myrow;
-    int_t  i, ii, il, j, k, lb, ljb, lk, lptr, luptr;
+    int    i, ii, j, k, lb, ljb, lk;
+    int_t  il, lptr, luptr;
     int_t  nb, nlb, nub, nsupers;
     int_t  *xsup, *lsub, *usub;
     int_t  *ilsum;    /* Starting position of each supernode in lsum (LOCAL)*/
@@ -125,19 +131,19 @@ void psgstrs1(int_t n, sLUstruct_t *LUstruct, gridinfo_t *grid,
 #endif
 
     /*-- Counts used for L-solve --*/
-    int_t  *fmod;         /* Modification count for L-solve. */
-    int_t  **fsendx_plist = Llu->fsendx_plist;
-    int_t  nfrecvx = Llu->nfrecvx; /* Number of X components to be recv'd. */
-    int_t  *frecv;        /* Count of modifications to be recv'd from
+    int  *fmod;         /* Modification count for L-solve. */
+    int  **fsendx_plist = Llu->fsendx_plist;
+    int  nfrecvx = Llu->nfrecvx; /* Number of X components to be recv'd. */
+    int  *frecv;        /* Count of modifications to be recv'd from
 			     processes in this row. */
-    int_t  nfrecvmod = 0; /* Count of total modifications to be recv'd. */
-    int_t  nleaf = 0, nroot = 0;
+    int nfrecvmod = 0; /* Count of total modifications to be recv'd. */
+    int nleaf = 0, nroot = 0;
 
     /*-- Counts used for U-solve --*/
-    int_t  *bmod;         /* Modification count for L-solve. */
-    int_t  **bsendx_plist = Llu->bsendx_plist;
-    int_t  nbrecvx = Llu->nbrecvx; /* Number of X components to be recv'd. */
-    int_t  *brecv;        /* Count of modifications to be recv'd from
+    int  *bmod;         /* Modification count for L-solve. */
+    int  **bsendx_plist = Llu->bsendx_plist;
+    int  nbrecvx = Llu->nbrecvx; /* Number of X components to be recv'd. */
+    int  *brecv;        /* Count of modifications to be recv'd from
 			     processes in this row. */
     int_t  nbrecvmod = 0; /* Count of total modifications to be recv'd. */
     double t;
@@ -145,7 +151,7 @@ void psgstrs1(int_t n, sLUstruct_t *LUstruct, gridinfo_t *grid,
     int_t Ublocks = 0;
 #endif
 
-    int_t *mod_bit = Llu->mod_bit; /* flag contribution from each row block */
+    int *mod_bit = Llu->mod_bit; /* flag contribution from each row block */
 
     t = SuperLU_timer_();
 
@@ -179,10 +185,10 @@ void psgstrs1(int_t n, sLUstruct_t *LUstruct, gridinfo_t *grid,
 
     /* Save the count to be altered so it can be used by
        subsequent call to PSGSTRS1. */
-    if ( !(fmod = intMalloc_dist(nlb)) )
+    if ( !(fmod = int32Malloc_dist(nlb)) )
 	ABORT("Calloc fails for fmod[].");
     for (i = 0; i < nlb; ++i) fmod[i] = Llu->fmod[i];
-    if ( !(frecv = intMalloc_dist(nlb)) )
+    if ( !(frecv = int32Malloc_dist(nlb)) )
 	ABORT("Malloc fails for frecv[].");
     Llu->frecv = frecv;
 
@@ -204,7 +210,7 @@ void psgstrs1(int_t n, sLUstruct_t *LUstruct, gridinfo_t *grid,
     ldalsum = Llu->ldalsum;
 
     /* Allocate working storage. */
-    knsupc = sp_ienv_dist(3);
+    knsupc = sp_ienv_dist(3, options);
     if ( !(lsum = floatCalloc_dist(((size_t)ldalsum) * nrhs
         + nlb * LSUM_H)) )
 	ABORT("Calloc fails for lsum[].");
@@ -249,11 +255,12 @@ void psgstrs1(int_t n, sLUstruct_t *LUstruct, gridinfo_t *grid,
 		    mod_bit[lk] = 1;  /* contribution from off-diagonal */
 	    }
 	}
-	/*PrintInt10("mod_bit", nlb, mod_bit);*/
+	/*PrintInt32("mod_bit", nlb, mod_bit);*/
 
 	/* Every process receives the count, but it is only useful on the
 	   diagonal processes.  */
-	MPI_Allreduce( mod_bit, frecv, nlb, mpi_int_t, MPI_SUM, scp->comm );
+	//MPI_Allreduce( mod_bit, frecv, nlb, mpi_int_t, MPI_SUM, scp->comm );
+	MPI_Allreduce( mod_bit, frecv, nlb, MPI_INT, MPI_SUM, scp->comm );
 
 	for (k = 0; k < nsupers; ++k) {
 	    krow = PROW( k, grid );
@@ -528,10 +535,10 @@ void psgstrs1(int_t n, sLUstruct_t *LUstruct, gridinfo_t *grid,
 
     /* Save the count to be altered so it can be used by
        subsequent call to PSGSTRS1. */
-    if ( !(bmod = intMalloc_dist(nlb)) )
+    if ( !(bmod = int32Malloc_dist(nlb)) )
 	ABORT("Calloc fails for bmod[].");
     for (i = 0; i < nlb; ++i) bmod[i] = Llu->bmod[i];
-    if ( !(brecv = intMalloc_dist(nlb)) )
+    if ( !(brecv = int32Malloc_dist(nlb)) )
 	ABORT("Malloc fails for brecv[].");
     Llu->brecv = brecv;
 
@@ -555,7 +562,11 @@ void psgstrs1(int_t n, sLUstruct_t *LUstruct, gridinfo_t *grid,
 
 	/* Every process receives the count, but it is only useful on the
 	   diagonal processes.  */
+#if 0	   
 	MPI_Allreduce( mod_bit, brecv, nlb, mpi_int_t, MPI_SUM, scp->comm );
+#else	
+	MPI_Allreduce( mod_bit, brecv, nlb, MPI_INT, MPI_SUM, scp->comm );
+#endif
 
 	for (k = 0; k < nsupers; ++k) {
 	    krow = PROW( k, grid );
@@ -583,8 +594,13 @@ void psgstrs1(int_t n, sLUstruct_t *LUstruct, gridinfo_t *grid,
 		if ( mycol != kcol && bmod[lk] )
 		    i = 1;  /* Contribution from non-diagonal process. */
 		else i = 0;
+#if 0 // Sherry		
 		MPI_Reduce( &i, &brecv[lk], 1, mpi_int_t,
 			   MPI_SUM, kcol, scp->comm );
+#else			   
+		MPI_Reduce( &i, &brecv[lk], 1, MPI_INT, MPI_SUM, kcol, scp->comm );
+#endif
+
 		if ( mycol == kcol ) { /* Diagonal process. */
 		    nbrecvmod += brecv[lk];
 		    if ( !brecv[lk] && !bmod[lk] ) ++nroot;
