@@ -454,7 +454,7 @@ sDestroy_LU(int_t n, gridinfo_t *grid, sLUstruct_t *LUstruct)
 	    SUPERLU_FREE (Llu->Lrowind_bc_ptr[i]);
 #if 0 // Sherry: the following is not allocated with cudaHostAlloc    
     //#ifdef GPU_ACC
-	    checkCuda(cudaFreeHost(Llu->Lnzval_bc_ptr[i]));
+	    checkGPU(gpuFreeHost(Llu->Lnzval_bc_ptr[i]));
 #endif
 	    SUPERLU_FREE (Llu->Lnzval_bc_ptr[i]);
 	}
@@ -479,10 +479,10 @@ sDestroy_LU(int_t n, gridinfo_t *grid, sLUstruct_t *LUstruct)
     /* The following can be freed only after iterative refinement. */
     SUPERLU_FREE(Llu->ilsum);
     SUPERLU_FREE(Llu->fmod);
-    SUPERLU_FREE(Llu->fsendx_plist[0]);
+    SUPERLU_FREE((Llu->fsendx_plist)[0]);
     SUPERLU_FREE(Llu->fsendx_plist);
     SUPERLU_FREE(Llu->bmod);
-    SUPERLU_FREE(Llu->bsendx_plist[0]);
+    SUPERLU_FREE((Llu->bsendx_plist)[0]);
     SUPERLU_FREE(Llu->bsendx_plist);
     SUPERLU_FREE(Llu->mod_bit);
 
@@ -827,25 +827,35 @@ void psinf_norm_error(int iam, int_t n, int_t nrhs, float x[], int_t ldx,
 {
     float err, xnorm, temperr, tempxnorm;
     float *x_work, *xtrue_work;
+    float errcomp;  // componentwise error
+    double derr;
     int i, j;
 
     for (j = 0; j < nrhs; j++) {
       x_work = &x[j*ldx];
       xtrue_work = &xtrue[j*ldxtrue];
-      err = xnorm = 0.0;
+      err = xnorm = errcomp = 0.0;
       for (i = 0; i < n; i++) {
-	err = SUPERLU_MAX(err, fabs(x_work[i] - xtrue_work[i]));
+	derr = fabs(x_work[i] - xtrue_work[i]);
+	err = SUPERLU_MAX(err, derr);
 	xnorm = SUPERLU_MAX(xnorm, fabs(x_work[i]));
+        errcomp = SUPERLU_MAX(errcomp, derr / fabs(x_work[i]));
       }
 
       /* get the golbal max err & xnrom */
       temperr = err;
-      tempxnorm = xnorm;
       MPI_Allreduce( &temperr, &err, 1, MPI_FLOAT, MPI_MAX, slucomm);
+      tempxnorm = xnorm;
       MPI_Allreduce( &tempxnorm, &xnorm, 1, MPI_FLOAT, MPI_MAX, slucomm);
+      temperr = errcomp;
+      MPI_Allreduce( &temperr, &errcomp, 1, MPI_FLOAT, MPI_MAX, slucomm);
 
       err = err / xnorm;
-      if ( !iam ) printf("\tSol %2d: ||X-Xtrue||/||X|| = %e\n", j, err);
+      if ( !iam ) {
+	printf(".. Sol %2d: ||X - Xtrue|| / ||X|| = %e\t max_i |x - xtrue|_i / |x|_i = %e\n", j, err, errcomp);
+	//printf("\t ||x||_inf = %e\n", xnorm);
+	fflush(stdout);
+      }
     }
 }
 
@@ -866,24 +876,28 @@ sDestroy_Tree(int_t n, gridinfo_t *grid, sLUstruct_t *LUstruct)
 
     nb = CEILING(nsupers, grid->npcol);
     for (i=0;i<nb;++i){
-	if(Llu->LBtree_ptr[i]!=NULL){
-		BcTree_Destroy(Llu->LBtree_ptr[i],LUstruct->dt);
+        if(Llu->LBtree_ptr[i].empty_==NO){    
+			// BcTree_Destroy(Llu->LBtree_ptr[i],LUstruct->dt);
+            C_BcTree_Nullify(&Llu->LBtree_ptr[i]);
+	}
+        if(Llu->UBtree_ptr[i].empty_==NO){  
+			// BcTree_Destroy(Llu->UBtree_ptr[i],LUstruct->dt);
+            C_BcTree_Nullify(&Llu->UBtree_ptr[i]);
 	}
-	if(Llu->UBtree_ptr[i]!=NULL){
-		BcTree_Destroy(Llu->UBtree_ptr[i],LUstruct->dt);
-	}		
     }
     SUPERLU_FREE(Llu->LBtree_ptr);
     SUPERLU_FREE(Llu->UBtree_ptr);
 	
     nb = CEILING(nsupers, grid->nprow);
     for (i=0;i<nb;++i){
-	if(Llu->LRtree_ptr[i]!=NULL){
-		RdTree_Destroy(Llu->LRtree_ptr[i],LUstruct->dt);
+        if(Llu->LRtree_ptr[i].empty_==NO){             
+			// RdTree_Destroy(Llu->LRtree_ptr[i],LUstruct->dt);
+            C_RdTree_Nullify(&Llu->LRtree_ptr[i]);
+	}
+        if(Llu->URtree_ptr[i].empty_==NO){ 
+			// RdTree_Destroy(Llu->URtree_ptr[i],LUstruct->dt);
+            C_RdTree_Nullify(&Llu->URtree_ptr[i]);
 	}
-	if(Llu->URtree_ptr[i]!=NULL){
-		RdTree_Destroy(Llu->URtree_ptr[i],LUstruct->dt);
-	}		
     }
     SUPERLU_FREE(Llu->LRtree_ptr);
     SUPERLU_FREE(Llu->URtree_ptr);
