@@ -41,7 +41,7 @@ _fcd ftcs2;
 _fcd ftcs3;
 #endif
 static void gather_diag_to_all(int_t, int_t, doublecomplex [], Glu_persist_t *,
-                               LocalLU_t *, gridinfo_t *, int_t, int_t [],
+                               zLocalLU_t *, gridinfo_t *, int_t, int_t [],
                                int_t [], doublecomplex [], int_t, doublecomplex []);
 
 /*! \brief
@@ -57,14 +57,18 @@ static void gather_diag_to_all(int_t, int_t, doublecomplex [], Glu_persist_t *,
  * Arguments
  * =========
  *
+ * options (input) superlu_dist_options_t*
+ *         The structure defines the input parameters to control
+ *         how the LU decomposition and triangular solve are performed.
+ *
  * n      (input) int (global)
  *        The order of the system of linear equations.
  *
- * LUstruct (input) LUstruct_t*
+ * LUstruct (input) zLUstruct_t*
  *        The distributed data structures storing L and U factors.
  *        The L and U factors are obtained from pzgstrf for
  *        the possibly scaled and permuted matrix A.
- *        See superlu_ddefs.h for the definition of 'LUstruct_t'.
+ *        See superlu_ddefs.h for the definition of 'zLUstruct_t'.
  *
  * grid   (input) gridinfo_t*
  *        The 2D process mesh. It contains the MPI communicator, the number
@@ -100,12 +104,13 @@ static void gather_diag_to_all(int_t, int_t, doublecomplex [], Glu_persist_t *,
  */
 
 void
-pzgstrs_Bglobal(int_t n, LUstruct_t *LUstruct, gridinfo_t *grid,
+pzgstrs_Bglobal(superlu_dist_options_t *options, int_t n,
+                zLUstruct_t *LUstruct, gridinfo_t *grid,
                 doublecomplex *B, int_t ldb, int nrhs,
                 SuperLUStat_t *stat, int *info)
 {
     Glu_persist_t *Glu_persist = LUstruct->Glu_persist;
-    LocalLU_t *Llu = LUstruct->Llu;
+    zLocalLU_t *Llu = LUstruct->Llu;
     doublecomplex alpha = {1.0, 0.0};
     doublecomplex zero = {0.0, 0.0};
     doublecomplex *lsum;  /* Local running sum of the updates to B-components */
@@ -134,19 +139,19 @@ pzgstrs_Bglobal(int_t n, LUstruct_t *LUstruct, gridinfo_t *grid,
 #endif
 
     /*-- Counts used for L-solve --*/
-    int_t  *fmod;         /* Modification count for L-solve. */
-    int_t  **fsendx_plist = Llu->fsendx_plist;
-    int_t  nfrecvx = Llu->nfrecvx; /* Number of X components to be recv'd. */
-    int_t  *frecv;        /* Count of modifications to be recv'd from
+    int  *fmod;         /* Modification count for L-solve. */
+    int  **fsendx_plist = Llu->fsendx_plist;
+    int  nfrecvx = Llu->nfrecvx; /* Number of X components to be recv'd. */
+    int  *frecv;        /* Count of modifications to be recv'd from
 			     processes in this row. */
-    int_t  nfrecvmod = 0; /* Count of total modifications to be recv'd. */
-    int_t  nleaf = 0, nroot = 0;
+    int  nfrecvmod = 0; /* Count of total modifications to be recv'd. */
+    int  nleaf = 0, nroot = 0;
 
     /*-- Counts used for U-solve --*/
-    int_t  *bmod;         /* Modification count for L-solve. */
-    int_t  **bsendx_plist = Llu->bsendx_plist;
-    int_t  nbrecvx = Llu->nbrecvx; /* Number of X components to be recv'd. */
-    int_t  *brecv;        /* Count of modifications to be recv'd from
+    int  *bmod;         /* Modification count for L-solve. */
+    int  **bsendx_plist = Llu->bsendx_plist;
+    int  nbrecvx = Llu->nbrecvx; /* Number of X components to be recv'd. */
+    int  *brecv;        /* Count of modifications to be recv'd from
 			     processes in this row. */
     int_t  nbrecvmod = 0; /* Count of total modifications to be recv'd. */
     double t;
@@ -154,7 +159,7 @@ pzgstrs_Bglobal(int_t n, LUstruct_t *LUstruct, gridinfo_t *grid,
     int_t Ublocks = 0;
 #endif
 
-    int_t *mod_bit = Llu->mod_bit; /* flag contribution from each row block */
+    int *mod_bit = Llu->mod_bit; /* flag contribution from each row block */
 
     t = SuperLU_timer_();
 
@@ -189,10 +194,10 @@ pzgstrs_Bglobal(int_t n, LUstruct_t *LUstruct, gridinfo_t *grid,
 
     /* Save the count to be altered so it can be used by
        subsequent call to PDGSTRS_BGLOBAL. */
-    if ( !(fmod = intMalloc_dist(nlb)) )
+    if ( !(fmod = int32Malloc_dist(nlb)) )
 	ABORT("Calloc fails for fmod[].");
     for (i = 0; i < nlb; ++i) fmod[i] = Llu->fmod[i];
-    if ( !(frecv = intMalloc_dist(nlb)) )
+    if ( !(frecv = int32Malloc_dist(nlb)) )
 	ABORT("Malloc fails for frecv[].");
     Llu->frecv = frecv;
 
@@ -214,7 +219,7 @@ pzgstrs_Bglobal(int_t n, LUstruct_t *LUstruct, gridinfo_t *grid,
     ldalsum = Llu->ldalsum;
 
     /* Allocate working storage. */
-    knsupc = sp_ienv_dist(3);
+    knsupc = sp_ienv_dist(3, options);
     maxrecvsz = knsupc * nrhs + SUPERLU_MAX( XK_H, LSUM_H );
     if ( !(lsum = doublecomplexCalloc_dist(((size_t)ldalsum) * nrhs
         + nlb * LSUM_H)) )
@@ -277,7 +282,11 @@ pzgstrs_Bglobal(int_t n, LUstruct_t *LUstruct, gridinfo_t *grid,
 
 	/* Every process receives the count, but it is only useful on the
 	   diagonal processes.  */
+#if 0	   
 	MPI_Allreduce( mod_bit, frecv, nlb, mpi_int_t, MPI_SUM, scp->comm );
+#else	
+	MPI_Allreduce( mod_bit, frecv, nlb, MPI_INT, MPI_SUM, scp->comm );
+#endif	
 
 	for (k = 0; k < nsupers; ++k) {
 	    krow = PROW( k, grid );
@@ -570,10 +579,10 @@ pzgstrs_Bglobal(int_t n, LUstruct_t *LUstruct, gridinfo_t *grid,
 
     /* Save the count to be altered so it can be used by
        subsequent call to PDGSTRS_BGLOBAL. */
-    if ( !(bmod = intMalloc_dist(nlb)) )
+    if ( !(bmod = int32Malloc_dist(nlb)) )
 	ABORT("Calloc fails for bmod[].");
     for (i = 0; i < nlb; ++i) bmod[i] = Llu->bmod[i];
-    if ( !(brecv = intMalloc_dist(nlb)) )
+    if ( !(brecv = int32Malloc_dist(nlb)) )
 	ABORT("Malloc fails for brecv[].");
     Llu->brecv = brecv;
 
@@ -597,7 +606,11 @@ pzgstrs_Bglobal(int_t n, LUstruct_t *LUstruct, gridinfo_t *grid,
 
 	/* Every process receives the count, but it is only useful on the
 	   diagonal processes.  */
+#if 0	   
 	MPI_Allreduce( mod_bit, brecv, nlb, mpi_int_t, MPI_SUM, scp->comm );
+#else	
+	MPI_Allreduce( mod_bit, brecv, nlb, MPI_INT, MPI_SUM, scp->comm );
+#endif
 
 	for (k = 0; k < nsupers; ++k) {
 	    krow = PROW( k, grid );
@@ -998,7 +1011,7 @@ pzgstrs_Bglobal(int_t n, LUstruct_t *LUstruct, gridinfo_t *grid,
  */
 static void
 gather_diag_to_all(int_t n, int_t nrhs, doublecomplex x[],
-		   Glu_persist_t *Glu_persist, LocalLU_t *Llu,
+		   Glu_persist_t *Glu_persist, zLocalLU_t *Llu,
 		   gridinfo_t *grid, int_t num_diag_procs,
 		   int_t diag_procs[], int_t diag_len[],
 		   doublecomplex y[], int_t ldy, doublecomplex work[])
