@@ -96,7 +96,8 @@ int
 sldperm_dist(int job, int n, int_t nnz, int_t colptr[], int_t adjncy[],
 	float nzval[], int_t *perm, float u[], float v[])
 {
-    int i, num, icntl[10], info[10]; 
+    int i;
+    int num, icntl[10], info[10]; 
     int_t liw, ldw;
     int_t *iw;
     double *dw;
