@@ -9,14 +9,17 @@
 /* Enable parmetis */
 #define HAVE_PARMETIS TRUE
 
+/* Enable colamd */
+/* #undef HAVE_COLAMD */
+
 /* Enable LAPACK */
 /* #undef SLU_HAVE_LAPACK */
 
 /* Enable CombBLAS */
-#define HAVE_COMBBLAS TRUE
+/* #undef HAVE_COMBBLAS */
 
 /* enable 64bit index mode */
-/* #undef XSDK_INDEX_SIZE */
+#define XSDK_INDEX_SIZE 64
 
 #if (XSDK_INDEX_SIZE == 64)
 #define _LONGINT 1
