@@ -63,15 +63,15 @@ void superlu_gridinit(MPI_Comm Bcomm, /* The base communicator upon which
     SUPERLU_FREE(usermap);
     
 #ifdef GPU_ACC
-    /* Binding each MPI to a CUDA device */
+    /* Binding each MPI to a GPU device */
     char *ttemp;
     ttemp = getenv ("SUPERLU_BIND_MPI_GPU");
 
     if (ttemp) {
 	int devs, rank;
 	MPI_Comm_rank(Bcomm, &rank); // MPI_COMM_WORLD??
-	cudaGetDeviceCount(&devs);  // Returns the number of compute-capable devices
-	cudaSetDevice(rank % devs); // Set device to be used for GPU executions
+	gpuGetDeviceCount(&devs);  // Returns the number of compute-capable devices
+	gpuSetDevice(rank % devs); // Set device to be used for GPU executions
     }
 #endif
 }
