@@ -96,19 +96,46 @@ typedef struct {
 #define MAX_LOOKAHEADS 50
 typedef struct {
     int_t   **Lrowind_bc_ptr; /* size ceil(NSUPERS/Pc)                 */
+    int_t *Lrowind_bc_dat;  /* size sum of sizes of Lrowind_bc_ptr[lk])    */   
+    long int *Lrowind_bc_offset;  /* size ceil(NSUPERS/Pc)                 */     
+    long int Lrowind_bc_cnt;
+
     doublecomplex **Lnzval_bc_ptr;  /* size ceil(NSUPERS/Pc)                 */
+    doublecomplex *Lnzval_bc_dat;  /* size sum of sizes of Lnzval_bc_ptr[lk])  */   
+    long int *Lnzval_bc_offset;  /* size ceil(NSUPERS/Pc)                */    
+    long int Lnzval_bc_cnt;
+    
     doublecomplex **Linv_bc_ptr;  /* size ceil(NSUPERS/Pc)                 */
+    doublecomplex *Linv_bc_dat;  /* size sum of sizes of Linv_bc_ptr[lk])  */   
+    long int *Linv_bc_offset;  /* size ceil(NSUPERS/Pc)              */   
+    long int Linv_bc_cnt;
+    
     int_t   **Lindval_loc_bc_ptr; /* size ceil(NSUPERS/Pc)  pointers to locations in Lrowind_bc_ptr and Lnzval_bc_ptr */
+    int_t *Lindval_loc_bc_dat;  /* size sum of sizes of Lindval_loc_bc_ptr[lk]) */   
+    long int *Lindval_loc_bc_offset;  /* size ceil(NSUPERS/Pc)                  */   
+    long int Lindval_loc_bc_cnt;  
     int_t   *Unnz; /* number of nonzeros per block column in U*/
-	int_t   **Lrowind_bc_2_lsum; /* size ceil(NSUPERS/Pc)  map indices of Lrowind_bc_ptr to indices of lsum  */
-    doublecomplex  **Uinv_bc_ptr;  /* size ceil(NSUPERS/Pc)     	*/
+    int_t   **Lrowind_bc_2_lsum; /* size ceil(NSUPERS/Pc)  map indices of Lrowind_bc_ptr to indices of lsum  */
+    doublecomplex **Uinv_bc_ptr;  /* size ceil(NSUPERS/Pc)     	*/
+    doublecomplex *Uinv_bc_dat;  /* size sum of sizes of Linv_bc_ptr[lk])                 */   
+    long int *Uinv_bc_offset;  /* size ceil(NSUPERS/Pc)                 */   
+    long int Uinv_bc_cnt;
+
     int_t   **Ufstnz_br_ptr;  /* size ceil(NSUPERS/Pr)                 */
-    doublecomplex  **Unzval_br_ptr;  /* size ceil(NSUPERS/Pr)                 */
+    int_t   *Ufstnz_br_dat;  /* size sum of sizes of Ufstnz_br_ptr[lk])                 */   
+    long int *Ufstnz_br_offset;  /* size ceil(NSUPERS/Pr)    */
+    long int Ufstnz_br_cnt;
+    
+    doublecomplex  **Unzval_br_ptr;  /* size ceil(NSUPERS/Pr)                  */
+    doublecomplex  *Unzval_br_dat;   /* size sum of sizes of Unzval_br_ptr[lk]) */   
+    long int *Unzval_br_offset;  /* size ceil(NSUPERS/Pr)    */
+    long int Unzval_br_cnt;
+    
         /*-- Data structures used for broadcast and reduction trees. --*/
-    BcTree  *LBtree_ptr;       /* size ceil(NSUPERS/Pc)                */
-    RdTree  *LRtree_ptr;       /* size ceil(NSUPERS/Pr)                */
-    BcTree  *UBtree_ptr;       /* size ceil(NSUPERS/Pc)                */
-    RdTree  *URtree_ptr;       /* size ceil(NSUPERS/Pr)			*/
+    C_Tree  *LBtree_ptr;       /* size ceil(NSUPERS/Pc)                */
+    C_Tree  *LRtree_ptr;       /* size ceil(NSUPERS/Pr)                */
+    C_Tree  *UBtree_ptr;       /* size ceil(NSUPERS/Pc)                */
+    C_Tree  *URtree_ptr;       /* size ceil(NSUPERS/Pr)			*/
 #if 0
     int_t   *Lsub_buf;        /* Buffer for the remote subscripts of L */
     doublecomplex  *Lval_buf;        /* Buffer for the remote nonzeros of L   */
@@ -134,17 +161,18 @@ typedef struct {
     int   **ToSendR;        /* List of processes to send right block col. */
 
     /*-- Record communication schedule for forward/back solves. --*/
-    int_t   *fmod;            /* Modification count for L-solve            */
-    int_t   **fsendx_plist;   /* Column process list to send down Xk       */
-    int_t   *frecv;           /* Modifications to be recv'd in proc row    */
-    int_t   nfrecvx;          /* Number of Xk I will receive in L-solve    */
-    int_t   nfsendx;          /* Number of Xk I will send in L-solve       */
-    int_t   *bmod;            /* Modification count for U-solve            */
-    int_t   **bsendx_plist;   /* Column process list to send down Xk       */
-    int_t   *brecv;           /* Modifications to be recv'd in proc row    */
-    int_t   nbrecvx;          /* Number of Xk I will receive in U-solve    */
-    int_t   nbsendx;          /* Number of Xk I will send in U-solve       */
-    int_t   *mod_bit;         /* Flag contribution from each row blocks    */
+    /* 1/15/22 Sherry: changed int_t to int type */
+    int   *fmod;            /* Modification count for L-solve            */
+    int   **fsendx_plist;   /* Column process list to send down Xk       */
+    int   *frecv;           /* Modifications to be recv'd in proc row    */
+    int   nfrecvx;          /* Number of Xk I will receive in L-solve    */
+    int   nfsendx;          /* Number of Xk I will send in L-solve       */
+    int   *bmod;            /* Modification count for U-solve            */
+    int   **bsendx_plist;   /* Column process list to send down Xk       */
+    int   *brecv;           /* Modifications to be recv'd in proc row    */
+    int   nbrecvx;          /* Number of Xk I will receive in U-solve    */
+    int   nbsendx;          /* Number of Xk I will send in U-solve       */
+    int   *mod_bit;         /* Flag contribution from each row blocks    */
 
     /*-- Auxiliary arrays used for forward/back solves. --*/
     int_t   *ilsum;           /* Starting position of each supernode in lsum
@@ -173,13 +201,42 @@ typedef struct {
     int_t *ut_modbit;
     int_t *Urbs;
     Ucb_indptr_t **Ucb_indptr;/* Vertical linked list pointing to Uindex[] */
+    Ucb_indptr_t *Ucb_inddat;
+    long int *Ucb_indoffset;
+    long int Ucb_indcnt;  
+    
     int_t  **Ucb_valptr;      /* Vertical linked list pointing to Unzval[] */
+    int_t  *Ucb_valdat;      
+    long int *Ucb_valoffset;
+    long int Ucb_valcnt;
 
     /* some additional counters for L solve */
     int_t n;
     int_t nleaf;
     int_t nfrecvmod;
     int_t inv; /* whether the diagonal block is inverted*/
+
+    /* The following variables are used in GPU trisolve*/
+#ifdef GPU_ACC
+    int_t *d_Lrowind_bc_dat;     
+    long int *d_Lrowind_bc_offset;      
+    doublecomplex *d_Lnzval_bc_dat;     
+    long int *d_Lnzval_bc_offset;     
+    doublecomplex *d_Linv_bc_dat ;     
+    doublecomplex *d_Uinv_bc_dat ;     
+    long int *d_Linv_bc_offset ;     
+    long int *d_Uinv_bc_offset ;     
+    int_t *d_Lindval_loc_bc_dat ;     
+    long int *d_Lindval_loc_bc_offset ;     
+
+    int_t  *d_ilsum ;
+    int_t *d_xsup ;
+    C_Tree  *d_LBtree_ptr ;
+    C_Tree  *d_LRtree_ptr ;
+    C_Tree  *d_UBtree_ptr ;
+    C_Tree  *d_URtree_ptr ;    
+#endif
+
 } zLocalLU_t;
 
 
@@ -236,6 +293,7 @@ typedef struct {
 
 // new structures for pdgstrf_4_8 
 
+#if 0  // Sherry: moved to superlu_defs.h
 typedef struct
 {
     int_t nub;
@@ -254,8 +312,6 @@ typedef struct
     int_t nsupr;
 } lPanelInfo_t;
 
- 
-
 /* HyP_t is the data structure to assist HALO offload of Schur-complement. */
 typedef struct
 {
@@ -280,11 +336,14 @@ typedef struct
 
     int_t buffer_size;
     int_t bigu_size;
-    int_t offloadCondition;
-    int_t superlu_acc_offload;
-    int_t nCudaStreams;
+    int offloadCondition;
+    int superlu_acc_offload;
+    int nGPUStreams;
 } HyP_t;
 
+#endif  // Above are moved to superlu_defs.h
+
+
 typedef struct 
 {
     int_t * Lsub_buf ;
@@ -311,7 +370,7 @@ typedef struct
     sForest_t** sForests;
     int_t* supernode2treeMap;
     zLUValSubBuf_t  *LUvsb;
-} trf3Dpartition_t;
+} ztrf3Dpartition_t;
 
 typedef struct
 {
@@ -325,15 +384,6 @@ typedef struct
     doublecomplex* BlockUFactor;
 } zdiagFactBufs_t;
 
-typedef struct
-{
-    Ublock_info_t* Ublock_info;
-    Remain_info_t*  Remain_info;
-    uPanelInfo_t* uPanelInfo;
-    lPanelInfo_t* lPanelInfo;
-} packLUInfo_t;
-
-//#endif
 /*=====================*/
 
 /***********************************************************************
@@ -411,13 +461,14 @@ extern int     sp_zgemv_dist (char *, doublecomplex, SuperMatrix *, doublecomple
 extern int     sp_zgemm_dist (char *, int, doublecomplex, SuperMatrix *,
                         doublecomplex *, int, doublecomplex, doublecomplex *, int);
 
-extern float zdistribute(fact_t, int_t, SuperMatrix *, Glu_freeable_t *,
+extern float zdistribute(superlu_dist_options_t *,
+                         int_t, SuperMatrix *, Glu_freeable_t *,
 			 zLUstruct_t *, gridinfo_t *);
 extern void  pzgssvx_ABglobal(superlu_dist_options_t *, SuperMatrix *,
 			      zScalePermstruct_t *, doublecomplex *,
 			      int, int, gridinfo_t *, zLUstruct_t *, double *,
 			      SuperLUStat_t *, int *);
-extern float pzdistribute(fact_t, int_t, SuperMatrix *,
+extern float pzdistribute(superlu_dist_options_t *, int_t, SuperMatrix *,
 			 zScalePermstruct_t *, Glu_freeable_t *,
 			 zLUstruct_t *, gridinfo_t *);
 extern void  pzgssvx(superlu_dist_options_t *, SuperMatrix *,
@@ -433,7 +484,7 @@ extern int_t pzgstrs_init(int_t, int_t, int_t, int_t,
                           int_t [], int_t [], gridinfo_t *grid,
 	                  Glu_persist_t *, zSOLVEstruct_t *);
 extern void pxgstrs_finalize(pxgstrs_comm_t *);
-extern int  zldperm_dist(int_t, int_t, int_t, int_t [], int_t [],
+extern int  zldperm_dist(int, int, int_t, int_t [], int_t [],
 		    doublecomplex [], int_t *, double [], double []);
 extern int  zstatic_schedule(superlu_dist_options_t *, int, int,
 		            zLUstruct_t *, gridinfo_t *, SuperLUStat_t *,
@@ -460,9 +511,11 @@ extern int_t pzgstrf(superlu_dist_options_t *, int, int, double anorm,
 #define IPM_PROF */
 
 /* Solve related */
-extern void pzgstrs_Bglobal(int_t, zLUstruct_t *, gridinfo_t *,
+extern void pzgstrs_Bglobal(superlu_dist_options_t *,
+                             int_t, zLUstruct_t *, gridinfo_t *,
 			     doublecomplex *, int_t, int, SuperLUStat_t *, int *);
-extern void pzgstrs(int_t, zLUstruct_t *, zScalePermstruct_t *, gridinfo_t *,
+extern void pzgstrs(superlu_dist_options_t *, int_t,
+                    zLUstruct_t *, zScalePermstruct_t *, gridinfo_t *,
 		    doublecomplex *, int_t, int_t, int_t, int, zSOLVEstruct_t *,
 		    SuperLUStat_t *, int *);
 extern void pzgstrf2_trsm(superlu_dist_options_t * options, int_t k0, int_t k,
@@ -476,36 +529,47 @@ extern int_t pzReDistribute_B_to_X(doublecomplex *B, int_t m_loc, int nrhs, int_
 				   zScalePermstruct_t *, Glu_persist_t *,
 				   gridinfo_t *, zSOLVEstruct_t *);
 extern void zlsum_fmod(doublecomplex *, doublecomplex *, doublecomplex *, doublecomplex *,
-		       int, int, int_t , int_t *, int_t, int_t, int_t,
+		       int, int, int_t , int *fmod, int_t, int_t, int_t,
 		       int_t *, gridinfo_t *, zLocalLU_t *,
 		       MPI_Request [], SuperLUStat_t *);
 extern void zlsum_bmod(doublecomplex *, doublecomplex *, doublecomplex *,
-                       int, int_t, int_t *, int_t *, Ucb_indptr_t **,
+                       int, int_t, int *bmod, int_t *, Ucb_indptr_t **,
                        int_t **, int_t *, gridinfo_t *, zLocalLU_t *,
 		       MPI_Request [], SuperLUStat_t *);
 
 extern void zlsum_fmod_inv(doublecomplex *, doublecomplex *, doublecomplex *, doublecomplex *,
-		       int, int_t , int_t *,
+		       int, int_t , int *fmod,
 		       int_t *, gridinfo_t *, zLocalLU_t *,
 		       SuperLUStat_t **, int_t *, int_t *, int_t, int_t, int_t, int_t, int, int);
 extern void zlsum_fmod_inv_master(doublecomplex *, doublecomplex *, doublecomplex *, doublecomplex *,
-		       int, int, int_t , int_t *, int_t,
+		       int, int, int_t , int *fmod, int_t,
 		       int_t *, gridinfo_t *, zLocalLU_t *,
 		       SuperLUStat_t **, int_t, int_t, int_t, int_t, int, int);
 extern void zlsum_bmod_inv(doublecomplex *, doublecomplex *, doublecomplex *, doublecomplex *,
-                       int, int_t, int_t *, int_t *, Ucb_indptr_t **,
+                       int, int_t, int *bmod, int_t *, Ucb_indptr_t **,
                        int_t **, int_t *, gridinfo_t *, zLocalLU_t *,
 		       SuperLUStat_t **, int_t *, int_t *, int_t, int_t, int, int);
 extern void zlsum_bmod_inv_master(doublecomplex *, doublecomplex *, doublecomplex *, doublecomplex *,
-                       int, int_t, int_t *, int_t *, Ucb_indptr_t **,
+                       int, int_t, int *bmod, int_t *, Ucb_indptr_t **,
                        int_t **, int_t *, gridinfo_t *, zLocalLU_t *,
 		       SuperLUStat_t **, int_t, int_t, int, int);
 
-extern void pzgsrfs(int_t, SuperMatrix *, double, zLUstruct_t *,
+extern void zComputeLevelsets(int , int_t , gridinfo_t *,
+		  Glu_persist_t *, zLocalLU_t *, int_t *);               
+			   
+#ifdef GPU_ACC               
+extern void zlsum_fmod_inv_gpu_wrap(int_t, int_t, int_t, int_t, doublecomplex *, doublecomplex *, int, int, int_t , int *fmod, C_Tree  *, C_Tree  *, int_t *, int_t *, int64_t *, doublecomplex *, int64_t *, doublecomplex *, int64_t *, int_t *, int64_t *, int_t *, gridinfo_t *, doublecomplex * , doublecomplex * , int_t );
+extern void zlsum_bmod_inv_gpu_wrap(superlu_dist_options_t *,
+int_t, int_t, int_t, int_t, doublecomplex *, doublecomplex *,int,int, int_t , int *bmod, C_Tree  *, C_Tree  *, int_t *, int_t *,int_t *, int64_t *, doublecomplex *, int64_t *, int_t  *, int64_t *, Ucb_indptr_t *, int64_t *, doublecomplex *, int64_t *,int_t *,gridinfo_t *);
+#endif
+
+extern void pzgsrfs(superlu_dist_options_t *, int_t,
+                    SuperMatrix *, double, zLUstruct_t *,
 		    zScalePermstruct_t *, gridinfo_t *,
 		    doublecomplex [], int_t, doublecomplex [], int_t, int,
 		    zSOLVEstruct_t *, double *, SuperLUStat_t *, int *);
-extern void pzgsrfs_ABXglobal(int_t, SuperMatrix *, double, zLUstruct_t *,
+extern void pzgsrfs_ABXglobal(superlu_dist_options_t *, int_t,
+                  SuperMatrix *, double, zLUstruct_t *,
 		  gridinfo_t *, doublecomplex *, int_t, doublecomplex *, int_t,
 		  int, double *, SuperLUStat_t *, int *);
 extern int   pzgsmv_AXglobal_setup(SuperMatrix *, Glu_persist_t *,
@@ -559,7 +623,7 @@ extern int  zread_binary(FILE *, int_t *, int_t *, int_t *,
 	                  doublecomplex **, int_t **, int_t **);
 
 /* Distribute the data for numerical factorization */
-extern float zdist_psymbtonum(fact_t, int_t, SuperMatrix *,
+extern float zdist_psymbtonum(superlu_dist_options_t *, int_t, SuperMatrix *,
                                 zScalePermstruct_t *, Pslu_freeable_t *,
                                 zLUstruct_t *, gridinfo_t *);
 extern void pzGetDiagU(int_t, zLUstruct_t *, gridinfo_t *, doublecomplex *);
@@ -578,6 +642,13 @@ extern int   file_zPrint_CompRowLoc_Matrix_dist(FILE *fp, SuperMatrix *A);
 extern void  PrintDoublecomplex(char *, int_t, doublecomplex *);
 extern int   file_PrintDoublecomplex(FILE *fp, char *, int_t, doublecomplex *);
 
+extern void zGenCOOLblocks(int, int_t, gridinfo_t*,
+		  Glu_persist_t*, zLocalLU_t *, int_t** , int_t** , doublecomplex ** , int_t* , int_t* );
+extern void zGenCSCLblocks(int, int_t, gridinfo_t*,
+		  Glu_persist_t*, zLocalLU_t *, doublecomplex **, int_t **, int_t **, int_t*, int_t*);
+extern void zGenCSRLblocks(int, int_t, gridinfo_t*,
+		  Glu_persist_t*, zLocalLU_t *, doublecomplex **, int_t **, int_t **, int_t*, int_t*);
+
 
 /* BLAS */
 
@@ -638,7 +709,6 @@ extern int superlu_ztrsv(char *uplo, char *trans, char *diag,
                   int n, doublecomplex *a, int lda, doublecomplex *x, int incx);
 
 #ifdef SLU_HAVE_LAPACK
-// LAPACK routine
 extern void ztrtri_(char*, char*, int*, doublecomplex*, int*, int*);
 #endif
 
@@ -662,7 +732,7 @@ extern void pzgssvx3d (superlu_dist_options_t *, SuperMatrix *,
 		       gridinfo3d_t *, zLUstruct_t *, zSOLVEstruct_t *, 
 		       double *berr, SuperLUStat_t *, int *info);
 extern int_t pzgstrf3d(superlu_dist_options_t *, int m, int n, double anorm,
-		       trf3Dpartition_t*, SCT_t *, zLUstruct_t *,
+		       ztrf3Dpartition_t*, SCT_t *, zLUstruct_t *,
 		       gridinfo3d_t *, SuperLUStat_t *, int *);
 extern void zInit_HyP(HyP_t* HyP, zLocalLU_t *Llu, int_t mcb, int_t mrb );
 extern void Free_HyP(HyP_t* HyP);
@@ -758,12 +828,12 @@ extern void zRgather_U(int_t k, int_t jj0, int_t *usub, doublecomplex *uval,
 		      int_t *iperm_c_supno, int_t *perm_u);
 
     /* from xtrf3Dpartition.h */
-extern trf3Dpartition_t* zinitTrf3Dpartition(int_t nsupers,
+extern ztrf3Dpartition_t* zinitTrf3Dpartition(int_t nsupers,
 					     superlu_dist_options_t *options,
 					     zLUstruct_t *LUstruct, gridinfo3d_t * grid3d);
-extern void zDestroy_trf3Dpartition(trf3Dpartition_t *trf3Dpartition, gridinfo3d_t *grid3d);
+extern void zDestroy_trf3Dpartition(ztrf3Dpartition_t *trf3Dpartition, gridinfo3d_t *grid3d);
 
-extern void z3D_printMemUse(trf3Dpartition_t*  trf3Dpartition,
+extern void z3D_printMemUse(ztrf3Dpartition_t*  trf3Dpartition,
 			    zLUstruct_t *LUstruct, gridinfo3d_t * grid3d);
 
 //extern int* getLastDep(gridinfo_t *grid, SuperLUStat_t *stat,
@@ -855,7 +925,7 @@ int_t zgatherFactoredLU(int_t sender, int_t receiver,
 
 /*Gathers all the L and U factors to grid 0 for solve stage 
 	By  repeatidly calling above function*/
-int_t zgatherAllFactoredLU(trf3Dpartition_t*  trf3Dpartition, zLUstruct_t* LUstruct,
+int_t zgatherAllFactoredLU(ztrf3Dpartition_t*  trf3Dpartition, zLUstruct_t* LUstruct,
 			   gridinfo3d_t* grid3d, SCT_t* SCT );
 
 /*Distributes data in each layer and initilizes ancestors
@@ -892,19 +962,13 @@ extern int_t zIrecv_LPanel (int_t k, int_t k0,  int_t* Lsub_buf,
 			    MPI_Request *, zLocalLU_t *, int);
 extern int_t zIrecv_UPanel(int_t k, int_t k0, int_t* Usub_buf, doublecomplex*,
 			   zLocalLU_t *, gridinfo_t*, MPI_Request *, int);
-extern int_t Wait_LSend(int_t k, gridinfo_t *grid, int **ToSendR,
-			MPI_Request *s, SCT_t*);
-extern int_t Wait_USend(MPI_Request *, gridinfo_t *, SCT_t *);
 extern int_t zWait_URecv(MPI_Request *, int* msgcnt, SCT_t *);
-extern int_t Check_LRecv(MPI_Request*, int* msgcnt);
 extern int_t zWait_LRecv(MPI_Request*, int* msgcnt, int* msgcntsU,
 			 gridinfo_t *, SCT_t*);
 extern int_t zISend_UDiagBlock(int_t k0, doublecomplex *ublk_ptr, int_t size,
 			       MPI_Request *, gridinfo_t *, int);
 extern int_t zRecv_UDiagBlock(int_t k0, doublecomplex *ublk_ptr, int_t size,
 			      int_t src, gridinfo_t *, SCT_t*, int);
-extern int_t Wait_UDiagBlockSend(MPI_Request *, gridinfo_t *, SCT_t *);
-extern int_t Wait_LDiagBlockSend(MPI_Request *, gridinfo_t *, SCT_t *);
 extern int_t zPackLBlock(int_t k, doublecomplex* Dest, Glu_persist_t *,
 			 gridinfo_t *, zLocalLU_t *);
 extern int_t zISend_LDiagBlock(int_t k0, doublecomplex *lblk_ptr, int_t size,
@@ -912,16 +976,12 @@ extern int_t zISend_LDiagBlock(int_t k0, doublecomplex *lblk_ptr, int_t size,
 extern int_t zIRecv_UDiagBlock(int_t k0, doublecomplex *ublk_ptr, int_t size,
 			       int_t src, MPI_Request *, gridinfo_t *,
 			       SCT_t*, int);
-extern int_t Wait_UDiagBlock_Recv(MPI_Request *, SCT_t *);
-extern int_t Test_UDiagBlock_Recv(MPI_Request *, SCT_t *);
 extern int_t zIRecv_LDiagBlock(int_t k0, doublecomplex *L_blk_ptr, int_t size,
 			       int_t src, MPI_Request *, gridinfo_t*, SCT_t*, int);
-extern int_t Wait_LDiagBlock_Recv(MPI_Request *, SCT_t *);
-extern int_t Test_LDiagBlock_Recv(MPI_Request *, SCT_t *);
-
 extern int_t zUDiagBlockRecvWait( int_t k,  int_t* IrecvPlcd_D, int_t* factored_L,
 				  MPI_Request *, gridinfo_t *, zLUstruct_t *, SCT_t *);
 extern int_t LDiagBlockRecvWait( int_t k, int_t* factored_U, MPI_Request *, gridinfo_t *);
+
 #if (MPI_VERSION>2)
 extern int_t zIBcast_UDiagBlock(int_t k, doublecomplex *ublk_ptr, int_t size,
 				MPI_Request *, gridinfo_t *);
@@ -940,8 +1000,6 @@ extern int_t zDiagFactIBCast(int_t k,  int_t k0,
 extern int_t zUPanelTrSolve( int_t k, doublecomplex* BlockLFactor, doublecomplex* bigV,
 			     int_t ldt, Ublock_info_t*, gridinfo_t *,
 			     zLUstruct_t *, SuperLUStat_t *, SCT_t *);
-extern int_t Wait_LUDiagSend(int_t k, MPI_Request *, MPI_Request *,
-			     gridinfo_t *, SCT_t *);
 extern int_t zLPanelUpdate(int_t k,  int_t* IrecvPlcd_D, int_t* factored_L,
 			   MPI_Request *, doublecomplex* BlockUFactor, gridinfo_t *,
 			   zLUstruct_t *, SCT_t *);
@@ -980,15 +1038,18 @@ extern int_t zSchurComplementSetupGPU(int_t k, msgs_t* msgs, packLUInfo_t*,
 				      zLUValSubBuf_t* LUvsb, gridinfo_t *,
 				      zLUstruct_t *, HyP_t*);
 extern doublecomplex* zgetBigV(int_t, int_t);
-extern doublecomplex* zgetBigU(int_t, gridinfo_t *, zLUstruct_t *);
+extern doublecomplex* zgetBigU(superlu_dist_options_t *,
+                           int_t, gridinfo_t *, zLUstruct_t *);
 // permutation from superLU default
 
     /* from treeFactorization.h */
 extern int_t zLluBufInit(zLUValSubBuf_t*, zLUstruct_t *);
-extern int_t zinitScuBufs(int_t ldt, int_t num_threads, int_t nsupers,
+extern int_t zinitScuBufs(superlu_dist_options_t *,
+                          int_t ldt, int_t num_threads, int_t nsupers,
 			  zscuBufs_t*, zLUstruct_t*, gridinfo_t *);
 extern int zfreeScuBufs(zscuBufs_t* scuBufs);
 
+#if 0 // NOT CALLED
 // the generic tree factoring code 
 extern int_t treeFactor(
     int_t nnnodes,          // number of nodes in the tree
@@ -1008,6 +1069,7 @@ extern int_t treeFactor(
     double thresh,  SCT_t *SCT,
     int *info
 );
+#endif
 
 extern int_t zsparseTreeFactor(
     int_t nnodes,          // number of nodes in the tree
@@ -1076,7 +1138,8 @@ extern int_t checkRecvUDiag(int_t k, commRequests_t *comReqs,
 			    gridinfo_t *grid, SCT_t *SCT);
 extern int_t checkRecvLDiag(int_t k, commRequests_t *comReqs, gridinfo_t *, SCT_t *);
 
-    /* from ancFactorization.h (not called) */
+#if 0 // NOT CALLED
+/* from ancFactorization.h (not called) */
 extern int_t ancestorFactor(
     int_t ilvl,             // level of factorization 
     sForest_t* sforest,
@@ -1096,6 +1159,7 @@ extern int_t ancestorFactor(
     zLUstruct_t *LUstruct, gridinfo3d_t * grid3d, SuperLUStat_t *stat,
     double thresh,  SCT_t *SCT, int tag_ub, int *info
 );
+#endif
 
 /*== end 3D prototypes ===================*/
 
